from typing import Optional
from pydantic import BaseModel
from kubemq.grpc import Result


class EventSendResult(BaseModel):
    id: Optional[str] = None
    sent: bool = False
    error: Optional[str] = None

    @classmethod
    def decode(cls, result: Result) -> "EventSendResult":
        return cls(id=result.EventID, sent=result.Sent, error=result.Error)

    def model_dump_json(self, **kwargs):
        return super().model_dump_json(exclude_none=True, **kwargs)

    class Config:
        validate_assignment = True



from .event_message import EventMessage
from .event_message_received import EventMessageReceived
from .event_send_result import EventSendResult
from .events_subscription import EventsSubscription
from .event_store_message import EventStoreMessage
from .event_store_message_received import EventStoreMessageReceived
from .events_store_subscription import EventsStoreSubscription, EventsStoreType
from .event_sender import EventSender
from kubemq.common import CancellationToken
from .client import Client



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# NO CHECKED-IN PROTOBUF GENCODE
# source: kubemq/grpc/kubemq.proto
# Protobuf Python Version: 5.29.0
"""Generated protocol buffer code."""

from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import runtime_version as _runtime_version
from google.protobuf import symbol_database as _symbol_database
from google.protobuf.internal import builder as _builder

_runtime_version.ValidateProtobufRuntimeVersion(
    _runtime_version.Domain.PUBLIC, 5, 29, 0, "", "kubemq/grpc/kubemq.proto"
)
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(
    b'\n\x18kubemq/grpc/kubemq.proto\x12\x06kubemq"a\n\nPingResult\x12\x0c\n\x04Host\x18\x01 \x01(\t\x12\x0f\n\x07Version\x18\x02 \x01(\t\x12\x17\n\x0fServerStartTime\x18\x03 \x01(\x03\x12\x1b\n\x13ServerUpTimeSeconds\x18\x04 \x01(\x03"\x07\n\x05\x45mpty"6\n\x06Result\x12\x0f\n\x07\x45ventID\x18\x01 \x01(\t\x12\x0c\n\x04Sent\x18\x02 \x01(\x08\x12\r\n\x05\x45rror\x18\x03 \x01(\t"\xbe\x01\n\x05\x45vent\x12\x0f\n\x07\x45ventID\x18\x01 \x01(\t\x12\x10\n\x08\x43lientID\x18\x02 \x01(\t\x12\x0f\n\x07\x43hannel\x18\x03 \x01(\t\x12\x10\n\x08Metadata\x18\x04 \x01(\t\x12\x0c\n\x04\x42ody\x18\x05 \x01(\x0c\x12\r\n\x05Store\x18\x06 \x01(\x08\x12%\n\x04Tags\x18\x07 \x03(\x0b\x32\x17.kubemq.Event.TagsEntry\x1a+\n\tTagsEntry\x12\x0b\n\x03key\x18\x01 \x01(\t\x12\r\n\x05value\x18\x02 \x01(\t:\x02\x38\x01"\xd0\x01\n\x0c\x45ventReceive\x12\x0f\n\x07\x45ventID\x18\x01 \x01(\t\x12\x0f\n\x07\x43hannel\x18\x02 \x01(\t\x12\x10\n\x08Metadata\x18\x03 \x01(\t\x12\x0c\n\x04\x42ody\x18\x04 \x01(\x0c\x12\x11\n\tTimestamp\x18\x05 \x01(\x03\x12\x10\n\x08Sequence\x18\x06 \x01(\x04\x12,\n\x04Tags\x18\x07 \x03(\x0b\x32\x1e.kubemq.EventReceive.TagsEntry\x1a+\n\tTagsEntry\x12\x0b\n\x03key\x18\x01 \x01(\t\x12\r\n\x05value\x18\x02 \x01(\t:\x02\x38\x01"\xe3\x03\n\tSubscribe\x12:\n\x11SubscribeTypeData\x18\x01 \x01(\x0e\x32\x1f.kubemq.Subscribe.SubscribeType\x12\x10\n\x08\x43lientID\x18\x02 \x01(\t\x12\x0f\n\x07\x43hannel\x18\x03 \x01(\t\x12\r\n\x05Group\x18\x04 \x01(\t\x12>\n\x13\x45ventsStoreTypeData\x18\x05 \x01(\x0e\x32!.kubemq.Subscribe.EventsStoreType\x12\x1c\n\x14\x45ventsStoreTypeValue\x18\x06 \x01(\x03"c\n\rSubscribeType\x12\x1a\n\x16SubscribeTypeUndefined\x10\x00\x12\n\n\x06\x45vents\x10\x01\x12\x0f\n\x0b\x45ventsStore\x10\x02\x12\x0c\n\x08\x43ommands\x10\x03\x12\x0b\n\x07Queries\x10\x04"\xa4\x01\n\x0f\x45ventsStoreType\x12\x1c\n\x18\x45ventsStoreTypeUndefined\x10\x00\x12\x10\n\x0cStartNewOnly\x10\x01\x12\x12\n\x0eStartFromFirst\x10\x02\x12\x11\n\rStartFromLast\x10\x03\x12\x13\n\x0fStartAtSequence\x10\x04\x12\x0f\n\x0bStartAtTime\x10\x05\x12\x14\n\x10StartAtTimeDelta\x10\x06"\x83\x03\n\x07Request\x12\x11\n\tRequestID\x18\x01 \x01(\t\x12\x34\n\x0fRequestTypeData\x18\x02 \x01(\x0e\x32\x1b.kubemq.Request.RequestType\x12\x10\n\x08\x43lientID\x18\x03 \x01(\t\x12\x0f\n\x07\x43hannel\x18\x04 \x01(\t\x12\x10\n\x08Metadata\x18\x05 \x01(\t\x12\x0c\n\x04\x42ody\x18\x06 \x01(\x0c\x12\x14\n\x0cReplyChannel\x18\x07 \x01(\t\x12\x0f\n\x07Timeout\x18\x08 \x01(\x05\x12\x10\n\x08\x43\x61\x63heKey\x18\t \x01(\t\x12\x10\n\x08\x43\x61\x63heTTL\x18\n \x01(\x05\x12\x0c\n\x04Span\x18\x0b \x01(\x0c\x12\'\n\x04Tags\x18\x0c \x03(\x0b\x32\x19.kubemq.Request.TagsEntry\x1a+\n\tTagsEntry\x12\x0b\n\x03key\x18\x01 \x01(\t\x12\r\n\x05value\x18\x02 \x01(\t:\x02\x38\x01"=\n\x0bRequestType\x12\x16\n\x12RequestTypeUnknown\x10\x00\x12\x0b\n\x07\x43ommand\x10\x01\x12\t\n\x05Query\x10\x02"\x90\x02\n\x08Response\x12\x10\n\x08\x43lientID\x18\x01 \x01(\t\x12\x11\n\tRequestID\x18\x02 \x01(\t\x12\x14\n\x0cReplyChannel\x18\x03 \x01(\t\x12\x10\n\x08Metadata\x18\x04 \x01(\t\x12\x0c\n\x04\x42ody\x18\x05 \x01(\x0c\x12\x10\n\x08\x43\x61\x63heHit\x18\x06 \x01(\x08\x12\x11\n\tTimestamp\x18\x07 \x01(\x03\x12\x10\n\x08\x45xecuted\x18\x08 \x01(\x08\x12\r\n\x05\x45rror\x18\t \x01(\t\x12\x0c\n\x04Span\x18\n \x01(\x0c\x12(\n\x04Tags\x18\x0b \x03(\x0b\x32\x1a.kubemq.Response.TagsEntry\x1a+\n\tTagsEntry\x12\x0b\n\x03key\x18\x01 \x01(\t\x12\r\n\x05value\x18\x02 \x01(\t:\x02\x38\x01"\xd7\x02\n\x0cQueueMessage\x12\x11\n\tMessageID\x18\x01 \x01(\t\x12\x10\n\x08\x43lientID\x18\x02 \x01(\t\x12\x0f\n\x07\x43hannel\x18\x03 \x01(\t\x12\x10\n\x08Metadata\x18\x04 \x01(\t\x12\x0c\n\x04\x42ody\x18\x05 \x01(\x0c\x12,\n\x04Tags\x18\x06 \x03(\x0b\x32\x1e.kubemq.QueueMessage.TagsEntry\x12\x32\n\nAttributes\x18\x07 \x01(\x0b\x32\x1e.kubemq.QueueMessageAttributes\x12*\n\x06Policy\x18\x08 \x01(\x0b\x32\x1a.kubemq.QueueMessagePolicy\x12\r\n\x05Topic\x18\t \x01(\t\x12\x11\n\tPartition\x18\n \x01(\x05\x12\x14\n\x0cPartitionKey\x18\x0b \x01(\t\x1a+\n\tTagsEntry\x12\x0b\n\x03key\x18\x01 \x01(\t\x12\r\n\x05value\x18\x02 \x01(\t:\x02\x38\x01"T\n\x19QueueMessagesBatchRequest\x12\x0f\n\x07\x42\x61tchID\x18\x01 \x01(\t\x12&\n\x08Messages\x18\x02 \x03(\x0b\x32\x14.kubemq.QueueMessage"r\n\x1aQueueMessagesBatchResponse\x12\x0f\n\x07\x42\x61tchID\x18\x01 \x01(\t\x12/\n\x07Results\x18\x02 \x03(\x0b\x32\x1e.kubemq.SendQueueMessageResult\x12\x12\n\nHaveErrors\x18\x03 \x01(\x08"\xbc\x01\n\x16QueueMessageAttributes\x12\x11\n\tTimestamp\x18\x01 \x01(\x03\x12\x10\n\x08Sequence\x18\x02 \x01(\x04\x12\x11\n\tMD5OfBody\x18\x03 \x01(\t\x12\x14\n\x0cReceiveCount\x18\x04 \x01(\x05\x12\x10\n\x08ReRouted\x18\x05 \x01(\x08\x12\x19\n\x11ReRoutedFromQueue\x18\x06 \x01(\t\x12\x14\n\x0c\x45xpirationAt\x18\x07 \x01(\x03\x12\x11\n\tDelayedTo\x18\x08 \x01(\x03"w\n\x12QueueMessagePolicy\x12\x19\n\x11\x45xpirationSeconds\x18\x01 \x01(\x05\x12\x14\n\x0c\x44\x65laySeconds\x18\x02 \x01(\x05\x12\x17\n\x0fMaxReceiveCount\x18\x03 \x01(\x05\x12\x17\n\x0fMaxReceiveQueue\x18\x04 \x01(\t"\xd1\x01\n\x16SendQueueMessageResult\x12\x11\n\tMessageID\x18\x01 \x01(\t\x12\x0e\n\x06SentAt\x18\x02 \x01(\x03\x12\x14\n\x0c\x45xpirationAt\x18\x03 \x01(\x03\x12\x11\n\tDelayedTo\x18\x04 \x01(\x03\x12\x0f\n\x07IsError\x18\x05 \x01(\x08\x12\r\n\x05\x45rror\x18\x06 \x01(\t\x12\x12\n\nRefChannel\x18\x07 \x01(\t\x12\x10\n\x08RefTopic\x18\x08 \x01(\t\x12\x14\n\x0cRefPartition\x18\t \x01(\x05\x12\x0f\n\x07RefHash\x18\n \x01(\t"\x99\x01\n\x1bReceiveQueueMessagesRequest\x12\x11\n\tRequestID\x18\x01 \x01(\t\x12\x10\n\x08\x43lientID\x18\x02 \x01(\t\x12\x0f\n\x07\x43hannel\x18\x03 \x01(\t\x12\x1b\n\x13MaxNumberOfMessages\x18\x04 \x01(\x05\x12\x17\n\x0fWaitTimeSeconds\x18\x05 \x01(\x05\x12\x0e\n\x06IsPeak\x18\x06 \x01(\x08"\xbc\x01\n\x1cReceiveQueueMessagesResponse\x12\x11\n\tRequestID\x18\x01 \x01(\t\x12&\n\x08Messages\x18\x02 \x03(\x0b\x32\x14.kubemq.QueueMessage\x12\x18\n\x10MessagesReceived\x18\x03 \x01(\x05\x12\x17\n\x0fMessagesExpired\x18\x04 \x01(\x05\x12\x0e\n\x06IsPeak\x18\x05 \x01(\x08\x12\x0f\n\x07IsError\x18\x06 \x01(\x08\x12\r\n\x05\x45rror\x18\x07 \x01(\t"k\n\x1a\x41\x63kAllQueueMessagesRequest\x12\x11\n\tRequestID\x18\x01 \x01(\t\x12\x10\n\x08\x43lientID\x18\x02 \x01(\t\x12\x0f\n\x07\x43hannel\x18\x03 \x01(\t\x12\x17\n\x0fWaitTimeSeconds\x18\x04 \x01(\x05"j\n\x1b\x41\x63kAllQueueMessagesResponse\x12\x11\n\tRequestID\x18\x01 \x01(\t\x12\x18\n\x10\x41\x66\x66\x65\x63tedMessages\x18\x02 \x01(\x04\x12\x0f\n\x07IsError\x18\x03 \x01(\x08\x12\r\n\x05\x45rror\x18\x04 \x01(\t"\x84\x02\n\x1aStreamQueueMessagesRequest\x12\x11\n\tRequestID\x18\x01 \x01(\t\x12\x10\n\x08\x43lientID\x18\x02 \x01(\t\x12\x38\n\x15StreamRequestTypeData\x18\x03 \x01(\x0e\x32\x19.kubemq.StreamRequestType\x12\x0f\n\x07\x43hannel\x18\x04 \x01(\t\x12\x19\n\x11VisibilitySeconds\x18\x05 \x01(\x05\x12\x17\n\x0fWaitTimeSeconds\x18\x06 \x01(\x05\x12\x13\n\x0bRefSequence\x18\x07 \x01(\x04\x12-\n\x0fModifiedMessage\x18\x08 \x01(\x0b\x32\x14.kubemq.QueueMessage"\xb1\x01\n\x1bStreamQueueMessagesResponse\x12\x11\n\tRequestID\x18\x01 \x01(\t\x12\x38\n\x15StreamRequestTypeData\x18\x02 \x01(\x0e\x32\x19.kubemq.StreamRequestType\x12%\n\x07Message\x18\x03 \x01(\x0b\x32\x14.kubemq.QueueMessage\x12\x0f\n\x07IsError\x18\x04 \x01(\x08\x12\r\n\x05\x45rror\x18\x05 \x01(\t"R\n\x15QueuesUpstreamRequest\x12\x11\n\tRequestID\x18\x01 \x01(\t\x12&\n\x08Messages\x18\x02 \x03(\x0b\x32\x14.kubemq.QueueMessage"\x7f\n\x16QueuesUpstreamResponse\x12\x14\n\x0cRefRequestID\x18\x01 \x01(\t\x12/\n\x07Results\x18\x02 \x03(\x0b\x32\x1e.kubemq.SendQueueMessageResult\x12\x0f\n\x07IsError\x18\x03 \x01(\x08\x12\r\n\x05\x45rror\x18\x04 \x01(\t"\x80\x03\n\x17QueuesDownstreamRequest\x12\x11\n\tRequestID\x18\x01 \x01(\t\x12\x10\n\x08\x43lientID\x18\x02 \x01(\t\x12<\n\x0fRequestTypeData\x18\x03 \x01(\x0e\x32#.kubemq.QueuesDownstreamRequestType\x12\x0f\n\x07\x43hannel\x18\x04 \x01(\t\x12\x10\n\x08MaxItems\x18\x05 \x01(\x05\x12\x13\n\x0bWaitTimeout\x18\x06 \x01(\x05\x12\x0f\n\x07\x41utoAck\x18\x07 \x01(\x08\x12\x16\n\x0eReQueueChannel\x18\x08 \x01(\t\x12\x15\n\rSequenceRange\x18\t \x03(\x03\x12\x18\n\x10RefTransactionId\x18\n \x01(\t\x12?\n\x08Metadata\x18\x0c \x03(\x0b\x32-.kubemq.QueuesDownstreamRequest.MetadataEntry\x1a/\n\rMetadataEntry\x12\x0b\n\x03key\x18\x01 \x01(\t\x12\r\n\x05value\x18\x02 \x01(\t:\x02\x38\x01"\xf4\x02\n\x18QueuesDownstreamResponse\x12\x15\n\rTransactionId\x18\x01 \x01(\t\x12\x14\n\x0cRefRequestId\x18\x02 \x01(\t\x12<\n\x0fRequestTypeData\x18\x03 \x01(\x0e\x32#.kubemq.QueuesDownstreamRequestType\x12&\n\x08Messages\x18\x04 \x03(\x0b\x32\x14.kubemq.QueueMessage\x12\x15\n\rActiveOffsets\x18\x05 \x03(\x03\x12\x0f\n\x07IsError\x18\x06 \x01(\x08\x12\r\n\x05\x45rror\x18\x07 \x01(\t\x12\x1b\n\x13TransactionComplete\x18\x08 \x01(\x08\x12@\n\x08Metadata\x18\t \x03(\x0b\x32..kubemq.QueuesDownstreamResponse.MetadataEntry\x1a/\n\rMetadataEntry\x12\x0b\n\x03key\x18\x01 \x01(\t\x12\r\n\x05value\x18\x02 \x01(\t:\x02\x38\x01"\xae\x01\n\tQueueInfo\x12\x0c\n\x04Name\x18\x01 \x01(\t\x12\x10\n\x08Messages\x18\x02 \x01(\x03\x12\r\n\x05\x42ytes\x18\x03 \x01(\x03\x12\x15\n\rFirstSequence\x18\x04 \x01(\x03\x12\x14\n\x0cLastSequence\x18\x05 \x01(\x03\x12\x0c\n\x04Sent\x18\x06 \x01(\x03\x12\x11\n\tDelivered\x18\x07 \x01(\x03\x12\x0f\n\x07Waiting\x18\x08 \x01(\x03\x12\x13\n\x0bSubscribers\x18\t \x01(\x03"u\n\nQueuesInfo\x12\x12\n\nTotalQueue\x18\x01 \x01(\x05\x12\x0c\n\x04Sent\x18\x02 \x01(\x03\x12\x11\n\tDelivered\x18\x03 \x01(\x03\x12\x0f\n\x07Waiting\x18\x04 \x01(\x03\x12!\n\x06Queues\x18\x05 \x03(\x0b\x32\x11.kubemq.QueueInfo"9\n\x11QueuesInfoRequest\x12\x11\n\tRequestID\x18\x01 \x01(\t\x12\x11\n\tQueueName\x18\x02 \x01(\t"L\n\x12QueuesInfoResponse\x12\x14\n\x0cRefRequestID\x18\x01 \x01(\t\x12 \n\x04Info\x18\x02 \x01(\x0b\x32\x12.kubemq.QueuesInfo*\xaa\x01\n\x11StreamRequestType\x12\x1c\n\x18StreamRequestTypeUnknown\x10\x00\x12\x12\n\x0eReceiveMessage\x10\x01\x12\x0e\n\nAckMessage\x10\x02\x12\x11\n\rRejectMessage\x10\x03\x12\x14\n\x10ModifyVisibility\x10\x04\x12\x11\n\rResendMessage\x10\x05\x12\x17\n\x13SendModifiedMessage\x10\x06*\xea\x01\n\x1bQueuesDownstreamRequestType\x12\x1a\n\x16PollRequestTypeUnknown\x10\x00\x12\x07\n\x03Get\x10\x01\x12\n\n\x06\x41\x63kAll\x10\x02\x12\x0c\n\x08\x41\x63kRange\x10\x03\x12\x0b\n\x07NAckAll\x10\x04\x12\r\n\tNAckRange\x10\x05\x12\x0e\n\nReQueueAll\x10\x06\x12\x10\n\x0cReQueueRange\x10\x07\x12\x11\n\rActiveOffsets\x10\x08\x12\x15\n\x11TransactionStatus\x10\t\x12\x11\n\rCloseByClient\x10\n\x12\x11\n\rCloseByServer\x10\x0b\x32\xda\x08\n\x06kubemq\x12,\n\tSendEvent\x12\r.kubemq.Event\x1a\x0e.kubemq.Result"\x00\x12\x37\n\x10SendEventsStream\x12\r.kubemq.Event\x1a\x0e.kubemq.Result"\x00(\x01\x30\x01\x12@\n\x11SubscribeToEvents\x12\x11.kubemq.Subscribe\x1a\x14.kubemq.EventReceive"\x00\x30\x01\x12=\n\x13SubscribeToRequests\x12\x11.kubemq.Subscribe\x1a\x0f.kubemq.Request"\x00\x30\x01\x12\x32\n\x0bSendRequest\x12\x0f.kubemq.Request\x1a\x10.kubemq.Response"\x00\x12\x31\n\x0cSendResponse\x12\x10.kubemq.Response\x1a\r.kubemq.Empty"\x00\x12J\n\x10SendQueueMessage\x12\x14.kubemq.QueueMessage\x1a\x1e.kubemq.SendQueueMessageResult"\x00\x12\x61\n\x16SendQueueMessagesBatch\x12!.kubemq.QueueMessagesBatchRequest\x1a".kubemq.QueueMessagesBatchResponse"\x00\x12\x63\n\x14ReceiveQueueMessages\x12#.kubemq.ReceiveQueueMessagesRequest\x1a$.kubemq.ReceiveQueueMessagesResponse"\x00\x12\x63\n\x12StreamQueueMessage\x12".kubemq.StreamQueueMessagesRequest\x1a#.kubemq.StreamQueueMessagesResponse"\x00(\x01\x30\x01\x12`\n\x13\x41\x63kAllQueueMessages\x12".kubemq.AckAllQueueMessagesRequest\x1a#.kubemq.AckAllQueueMessagesResponse"\x00\x12+\n\x04Ping\x12\r.kubemq.Empty\x1a\x12.kubemq.PingResult"\x00\x12[\n\x10QueuesDownstream\x12\x1f.kubemq.QueuesDownstreamRequest\x1a .kubemq.QueuesDownstreamResponse"\x00(\x01\x30\x01\x12U\n\x0eQueuesUpstream\x12\x1d.kubemq.QueuesUpstreamRequest\x1a\x1e.kubemq.QueuesUpstreamResponse"\x00(\x01\x30\x01\x12\x45\n\nQueuesInfo\x12\x19.kubemq.QueuesInfoRequest\x1a\x1a.kubemq.QueuesInfoResponse"\x00\x42\x10\xaa\x02\rKubeMQ.Clientb\x06proto3'
)

_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, "kubemq.grpc.kubemq_pb2", _globals)
if not _descriptor._USE_C_DESCRIPTORS:
    _globals["DESCRIPTOR"]._loaded_options = None
    _globals["DESCRIPTOR"]._serialized_options = b"\252\002\rKubeMQ.Client"
    _globals["_EVENT_TAGSENTRY"]._loaded_options = None
    _globals["_EVENT_TAGSENTRY"]._serialized_options = b"8\001"
    _globals["_EVENTRECEIVE_TAGSENTRY"]._loaded_options = None
    _globals["_EVENTRECEIVE_TAGSENTRY"]._serialized_options = b"8\001"
    _globals["_REQUEST_TAGSENTRY"]._loaded_options = None
    _globals["_REQUEST_TAGSENTRY"]._serialized_options = b"8\001"
    _globals["_RESPONSE_TAGSENTRY"]._loaded_options = None
    _globals["_RESPONSE_TAGSENTRY"]._serialized_options = b"8\001"
    _globals["_QUEUEMESSAGE_TAGSENTRY"]._loaded_options = None
    _globals["_QUEUEMESSAGE_TAGSENTRY"]._serialized_options = b"8\001"
    _globals["_QUEUESDOWNSTREAMREQUEST_METADATAENTRY"]._loaded_options = None
    _globals["_QUEUESDOWNSTREAMREQUEST_METADATAENTRY"]._serialized_options = b"8\001"
    _globals["_QUEUESDOWNSTREAMRESPONSE_METADATAENTRY"]._loaded_options = None
    _globals["_QUEUESDOWNSTREAMRESPONSE_METADATAENTRY"]._serialized_options = b"8\001"
    _globals["_STREAMREQUESTTYPE"]._serialized_start = 5243
    _globals["_STREAMREQUESTTYPE"]._serialized_end = 5413
    _globals["_QUEUESDOWNSTREAMREQUESTTYPE"]._serialized_start = 5416
    _globals["_QUEUESDOWNSTREAMREQUESTTYPE"]._serialized_end = 5650
    _globals["_PINGRESULT"]._serialized_start = 36
    _globals["_PINGRESULT"]._serialized_end = 133
    _globals["_EMPTY"]._serialized_start = 135
    _globals["_EMPTY"]._serialized_end = 142
    _globals["_RESULT"]._serialized_start = 144
    _globals["_RESULT"]._serialized_end = 198
    _globals["_EVENT"]._serialized_start = 201
    _globals["_EVENT"]._serialized_end = 391
    _globals["_EVENT_TAGSENTRY"]._serialized_start = 348
    _globals["_EVENT_TAGSENTRY"]._serialized_end = 391
    _globals["_EVENTRECEIVE"]._serialized_start = 394
    _globals["_EVENTRECEIVE"]._serialized_end = 602
    _globals["_EVENTRECEIVE_TAGSENTRY"]._serialized_start = 348
    _globals["_EVENTRECEIVE_TAGSENTRY"]._serialized_end = 391
    _globals["_SUBSCRIBE"]._serialized_start = 605
    _globals["_SUBSCRIBE"]._serialized_end = 1088
    _globals["_SUBSCRIBE_SUBSCRIBETYPE"]._serialized_start = 822
    _globals["_SUBSCRIBE_SUBSCRIBETYPE"]._serialized_end = 921
    _globals["_SUBSCRIBE_EVENTSSTORETYPE"]._serialized_start = 924
    _globals["_SUBSCRIBE_EVENTSSTORETYPE"]._serialized_end = 1088
    _globals["_REQUEST"]._serialized_start = 1091
    _globals["_REQUEST"]._serialized_end = 1478
    _globals["_REQUEST_TAGSENTRY"]._serialized_start = 348
    _globals["_REQUEST_TAGSENTRY"]._serialized_end = 391
    _globals["_REQUEST_REQUESTTYPE"]._serialized_start = 1417
    _globals["_REQUEST_REQUESTTYPE"]._serialized_end = 1478
    _globals["_RESPONSE"]._serialized_start = 1481
    _globals["_RESPONSE"]._serialized_end = 1753
    _globals["_RESPONSE_TAGSENTRY"]._serialized_start = 348
    _globals["_RESPONSE_TAGSENTRY"]._serialized_end = 391
    _globals["_QUEUEMESSAGE"]._serialized_start = 1756
    _globals["_QUEUEMESSAGE"]._serialized_end = 2099
    _globals["_QUEUEMESSAGE_TAGSENTRY"]._serialized_start = 348
    _globals["_QUEUEMESSAGE_TAGSENTRY"]._serialized_end = 391
    _globals["_QUEUEMESSAGESBATCHREQUEST"]._serialized_start = 2101
    _globals["_QUEUEMESSAGESBATCHREQUEST"]._serialized_end = 2185
    _globals["_QUEUEMESSAGESBATCHRESPONSE"]._serialized_start = 2187
    _globals["_QUEUEMESSAGESBATCHRESPONSE"]._serialized_end = 2301
    _globals["_QUEUEMESSAGEATTRIBUTES"]._serialized_start = 2304
    _globals["_QUEUEMESSAGEATTRIBUTES"]._serialized_end = 2492
    _globals["_QUEUEMESSAGEPOLICY"]._serialized_start = 2494
    _globals["_QUEUEMESSAGEPOLICY"]._serialized_end = 2613
    _globals["_SENDQUEUEMESSAGERESULT"]._serialized_start = 2616
    _globals["_SENDQUEUEMESSAGERESULT"]._serialized_end = 2825
    _globals["_RECEIVEQUEUEMESSAGESREQUEST"]._serialized_start = 2828
    _globals["_RECEIVEQUEUEMESSAGESREQUEST"]._serialized_end = 2981
    _globals["_RECEIVEQUEUEMESSAGESRESPONSE"]._serialized_start = 2984
    _globals["_RECEIVEQUEUEMESSAGESRESPONSE"]._serialized_end = 3172
    _globals["_ACKALLQUEUEMESSAGESREQUEST"]._serialized_start = 3174
    _globals["_ACKALLQUEUEMESSAGESREQUEST"]._serialized_end = 3281
    _globals["_ACKALLQUEUEMESSAGESRESPONSE"]._serialized_start = 3283
    _globals["_ACKALLQUEUEMESSAGESRESPONSE"]._serialized_end = 3389
    _globals["_STREAMQUEUEMESSAGESREQUEST"]._serialized_start = 3392
    _globals["_STREAMQUEUEMESSAGESREQUEST"]._serialized_end = 3652
    _globals["_STREAMQUEUEMESSAGESRESPONSE"]._serialized_start = 3655
    _globals["_STREAMQUEUEMESSAGESRESPONSE"]._serialized_end = 3832
    _globals["_QUEUESUPSTREAMREQUEST"]._serialized_start = 3834
    _globals["_QUEUESUPSTREAMREQUEST"]._serialized_end = 3916
    _globals["_QUEUESUPSTREAMRESPONSE"]._serialized_start = 3918
    _globals["_QUEUESUPSTREAMRESPONSE"]._serialized_end = 4045
    _globals["_QUEUESDOWNSTREAMREQUEST"]._serialized_start = 4048
    _globals["_QUEUESDOWNSTREAMREQUEST"]._serialized_end = 4432
    _globals["_QUEUESDOWNSTREAMREQUEST_METADATAENTRY"]._serialized_start = 4385
    _globals["_QUEUESDOWNSTREAMREQUEST_METADATAENTRY"]._serialized_end = 4432
    _globals["_QUEUESDOWNSTREAMRESPONSE"]._serialized_start = 4435
    _globals["_QUEUESDOWNSTREAMRESPONSE"]._serialized_end = 4807
    _globals["_QUEUESDOWNSTREAMRESPONSE_METADATAENTRY"]._serialized_start = 4385
    _globals["_QUEUESDOWNSTREAMRESPONSE_METADATAENTRY"]._serialized_end = 4432
    _globals["_QUEUEINFO"]._serialized_start = 4810
    _globals["_QUEUEINFO"]._serialized_end = 4984
    _globals["_QUEUESINFO"]._serialized_start = 4986
    _globals["_QUEUESINFO"]._serialized_end = 5103
    _globals["_QUEUESINFOREQUEST"]._serialized_start = 5105
    _globals["_QUEUESINFOREQUEST"]._serialized_end = 5162
    _globals["_QUEUESINFORESPONSE"]._serialized_start = 5164
    _globals["_QUEUESINFORESPONSE"]._serialized_end = 5240
    _globals["_KUBEMQ"]._serialized_start = 5653
    _globals["_KUBEMQ"]._serialized_end = 6767
# @@protoc_insertion_point(module_scope)



from pydantic import BaseModel, Field, field_validator, model_validator


class KeepAliveConfig(BaseModel):
    """
    Represents the configuration for keep alive functionality.
    """

    enabled: bool = Field(
        default=False, description="Specifies if keep alive is enabled"
    )
    ping_interval_in_seconds: int = Field(
        default=0, description="The interval at which ping requests are sent in seconds"
    )
    ping_timeout_in_seconds: int = Field(
        default=0, description="The timeout for ping requests in seconds"
    )

    @field_validator("ping_interval_in_seconds", "ping_timeout_in_seconds")
    def validate_positive_values(cls, v: int, info) -> int:
        if v < 0:
            raise ValueError(f"{info.field_name} must be greater than or equal to 0")
        return v

    @model_validator(mode="after")
    def validate_enabled_config(self) -> "KeepAliveConfig":
        if self.enabled:
            if self.ping_interval_in_seconds <= 0:
                raise ValueError(
                    "Keep alive ping interval must be greater than 0 when enabled"
                )
            if self.ping_timeout_in_seconds <= 0:
                raise ValueError(
                    "Keep alive ping timeout must be greater than 0 when enabled"
                )
        return self



from typing import Callable, Optional
from pydantic import BaseModel, field_validator
from kubemq.grpc import Subscribe
from kubemq.common.subscribe_type import SubscribeType
from kubemq.pubsub import EventMessageReceived


class EventsSubscription(BaseModel):
    channel: str
    group: Optional[str] = None
    on_receive_event_callback: Callable[[EventMessageReceived], None]
    on_error_callback: Optional[Callable[[str], None]] = None

    @field_validator("channel")
    def channel_must_exist(cls, v):
        if not v:
            raise ValueError("Event subscription must have a channel.")
        return v

    def raise_on_receive_message(self, received_event: EventMessageReceived):
        if self.on_receive_event_callback:
            self.on_receive_event_callback(received_event)

    def raise_on_error(self, msg: str):
        if self.on_error_callback:
            self.on_error_callback(msg)

    def encode(self, client_id: str = "") -> Subscribe:
        request = Subscribe()
        request.Channel = self.channel
        request.Group = self.group or ""
        request.ClientID = client_id
        request.SubscribeTypeData = SubscribeType.Events.value
        return request

    class Config:
        arbitrary_types_allowed = True

    def model_dump(self, **kwargs):
        dump = super().model_dump(**kwargs)
        # Remove callback functions from the dump
        dump.pop("on_receive_event_callback", None)
        dump.pop("on_error_callback", None)
        return dump



from kubemq.pubsub import *


def delete_events_channel(channelName):
    try:
        with Client(address="localhost:50000", client_id="delete_example") as client:
            client.delete_events_channel(channelName)
            print(f"Events channel: {channelName} deleted successfully.")
    except Exception as e:
        print(f"Error deleting Events Channel: {channelName},  {e}")


def delete_events_store_channel(channelName):
    try:
        with Client(address="localhost:50000", client_id="delete_example") as client:
            client.delete_events_store_channel(channelName)
            print(f"Events Store channel: {channelName} deleted successfully.")
    except Exception as e:
        print(f"Error creating Events Store Channel: {channelName},  {e}")


if __name__ == "__main__":
    delete_events_channel("new_events_channel")
    delete_events_store_channel("new_events_store_channel")



import asyncio
import logging
import os
from logging.config import dictConfig



from kubemq.queues import *

async def example_send_receive():
    with Client(
        address="localhost:50000",
    ) as client:
        async def _send():
            while True:
                send_result = await client.send_queues_message_async(
                    QueueMessage(
                        channel="send_receive",
                        body=b"message",
                    ),
                )
                if send_result.is_error:
                    print(f"Error sending message: {send_result.error}")
                    await asyncio.sleep(1)
                    continue
                print(f"Queue Message Sent: {send_result}")
                await asyncio.sleep(1)

        async def _receive():
            while True:
                send_receive_result = await client.receive_queues_messages_async(
                    channel="send_receive",
                    max_messages=1,
                    wait_timeout_in_seconds=1,
                )
                if send_receive_result.is_error:
                    print(f"Error receiving message: {send_receive_result.error}")
                    await asyncio.sleep(1)
                    continue
                for message in send_receive_result.messages:
                    print(f"Id:{message.id}, Body:{message.body.decode('utf-8')}")
                    message.ack()

                await asyncio.sleep(1)

        task1 = asyncio.create_task(_send())
        task2 = asyncio.create_task(_receive())
        tasks = [task1, task2]
        await asyncio.gather(*tasks)


if __name__ == "__main__":
    asyncio.run(example_send_receive())




from .command_message import CommandMessage
from .command_message_received import CommandMessageReceived
from .command_response_message import CommandResponseMessage
from .commands_subscription import CommandsSubscription
from .query_message import QueryMessage
from .query_message_received import QueryMessageReceived
from .query_response_message import QueryResponseMessage
from .queries_subscription import QueriesSubscription
from kubemq.common import CancellationToken
from .client import Client



from pydantic import BaseModel, Field, field_validator
from typing import Optional, Self
import uuid
from kubemq.grpc import QueuesDownstreamRequest, QueuesDownstreamRequestType


class QueuesPollRequest(BaseModel):
    """
    Class representing a request to poll messages from a queue.
    
    This class encapsulates the parameters needed to poll messages from a KubeMQ queue.
    It provides methods for validation and encoding to a protobuf request.
    
    Attributes:
        channel: The channel (queue name) to poll messages from.
        poll_max_messages: The maximum number of messages to poll in a single request.
        poll_wait_timeout_in_seconds: The maximum time to wait for messages in seconds.
        auto_ack_messages: Whether to automatically acknowledge received messages.
        visibility_seconds: The visibility time in seconds for the messages.
        
    Examples:
        ```python
        # Create a basic poll request
        request = QueuesPollRequest(
            channel="my-queue",
            poll_max_messages=10,
            poll_wait_timeout_in_seconds=30
        )
        
        # Create a poll request with auto acknowledgment
        request = QueuesPollRequest(
            channel="my-queue",
            poll_max_messages=5,
            poll_wait_timeout_in_seconds=10,
            auto_ack_messages=True
        )
        
        # Create a poll request with visibility timeout
        request = QueuesPollRequest(
            channel="my-queue",
            poll_max_messages=1,
            poll_wait_timeout_in_seconds=5,
            visibility_seconds=60  # Message will be invisible to other consumers for 60 seconds
        )
        
        # Encode the request to protobuf
        pb_request = request.encode("client-1")
        ```
    """
    
    # Pydantic configuration
    class Config:
        arbitrary_types_allowed = True
        frozen = True  # Make instances immutable

    # Instance attributes
    channel: Optional[str] = Field(
        default=None, description="The channel (queue name) to poll messages from"
    )
    poll_max_messages: int = Field(
        default=1,
        ge=1,
        description="The maximum number of messages to poll in a single request",
    )
    poll_wait_timeout_in_seconds: int = Field(
        default=60, 
        ge=1, 
        description="The maximum time to wait for messages in seconds"
    )
    auto_ack_messages: bool = Field(
        default=False,
        description="Whether to automatically acknowledge received messages",
    )
    visibility_seconds: int = Field(
        default=0,
        ge=0,
        description="The visibility time in seconds for the messages",
    )

    # Validators
    @field_validator("channel")
    def channel_must_not_be_empty(cls, v: Optional[str]) -> str:
        """
        Validate that the channel is not empty.
        
        Args:
            v: The channel value to validate
            
        Returns:
            The validated channel value
            
        Raises:
            ValueError: If the channel is empty
        """
        if not v:
            raise ValueError("Queue poll request must have a channel. Please provide a valid queue name.")
        return v
    
    @field_validator("visibility_seconds")
    def validate_visibility_seconds(cls, v: int) -> int:
        """
        Validate that the visibility seconds is within acceptable limits.
        
        Args:
            v: The visibility seconds value to validate
            
        Returns:
            The validated visibility seconds value
            
        Raises:
            ValueError: If the visibility seconds is negative
        """
        if v < 0:
            raise ValueError("Visibility seconds cannot be negative")
        return v

    # Utility methods
    def with_updates(self, **kwargs) -> Self:
        """
        Create a new poll request with updated values.
        
        Since instances are immutable, this method creates a new
        instance with the specified updates.
        
        Args:
            **kwargs: The fields to update and their new values
            
        Returns:
            A new instance with the updated values
        """
        data = self.model_dump()
        data.update(kwargs)
        return self.__class__(**data)
    
    # Encoding methods
    def encode(self, client_id: str = "") -> QueuesDownstreamRequest:
        """
        Encode the poll request to a QueuesDownstreamRequest protobuf object.
        
        This method is used when sending a poll request to the KubeMQ server.
        
        Args:
            client_id: The client ID to use for the request
            
        Returns:
            A QueuesDownstreamRequest protobuf object containing the encoded request
        """
        request = QueuesDownstreamRequest()
        request.RequestID = str(uuid.uuid4())
        request.ClientID = client_id
        request.Channel = self.channel
        request.MaxItems = self.poll_max_messages
        request.WaitTimeout = self.poll_wait_timeout_in_seconds * 1000
        request.AutoAck = self.auto_ack_messages
        request.RequestTypeData = QueuesDownstreamRequestType.Get
        return request

    # String representations
    def __str__(self) -> str:
        """
        Get a string representation of the poll request.
        
        Returns:
            A string representation of the poll request
        """
        return (
            f"QueuesPollRequest: channel={self.channel}, "
            f"poll_max_messages={self.poll_max_messages}, "
            f"poll_wait_timeout_in_seconds={self.poll_wait_timeout_in_seconds}, "
            f"auto_ack_messages={self.auto_ack_messages}, "
            f"visibility_seconds={self.visibility_seconds}"
        )
    
    def __repr__(self) -> str:
        """
        Get a detailed representation of the poll request.
        
        Returns:
            A detailed representation of the poll request
        """
        return (
            f"QueuesPollRequest(channel={self.channel!r}, "
            f"poll_max_messages={self.poll_max_messages}, "
            f"poll_wait_timeout_in_seconds={self.poll_wait_timeout_in_seconds}, "
            f"auto_ack_messages={self.auto_ack_messages}, "
            f"visibility_seconds={self.visibility_seconds})"
        )



from google.protobuf.internal import containers as _containers
from google.protobuf.internal import enum_type_wrapper as _enum_type_wrapper
from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from typing import (
    ClassVar as _ClassVar,
    Iterable as _Iterable,
    Mapping as _Mapping,
    Optional as _Optional,
    Union as _Union,
)

DESCRIPTOR: _descriptor.FileDescriptor

class StreamRequestType(int, metaclass=_enum_type_wrapper.EnumTypeWrapper):
    __slots__ = ()
    StreamRequestTypeUnknown: _ClassVar[StreamRequestType]
    ReceiveMessage: _ClassVar[StreamRequestType]
    AckMessage: _ClassVar[StreamRequestType]
    RejectMessage: _ClassVar[StreamRequestType]
    ModifyVisibility: _ClassVar[StreamRequestType]
    ResendMessage: _ClassVar[StreamRequestType]
    SendModifiedMessage: _ClassVar[StreamRequestType]

class QueuesDownstreamRequestType(int, metaclass=_enum_type_wrapper.EnumTypeWrapper):
    __slots__ = ()
    PollRequestTypeUnknown: _ClassVar[QueuesDownstreamRequestType]
    Get: _ClassVar[QueuesDownstreamRequestType]
    AckAll: _ClassVar[QueuesDownstreamRequestType]
    AckRange: _ClassVar[QueuesDownstreamRequestType]
    NAckAll: _ClassVar[QueuesDownstreamRequestType]
    NAckRange: _ClassVar[QueuesDownstreamRequestType]
    ReQueueAll: _ClassVar[QueuesDownstreamRequestType]
    ReQueueRange: _ClassVar[QueuesDownstreamRequestType]
    ActiveOffsets: _ClassVar[QueuesDownstreamRequestType]
    TransactionStatus: _ClassVar[QueuesDownstreamRequestType]
    CloseByClient: _ClassVar[QueuesDownstreamRequestType]
    CloseByServer: _ClassVar[QueuesDownstreamRequestType]

StreamRequestTypeUnknown: StreamRequestType
ReceiveMessage: StreamRequestType
AckMessage: StreamRequestType
RejectMessage: StreamRequestType
ModifyVisibility: StreamRequestType
ResendMessage: StreamRequestType
SendModifiedMessage: StreamRequestType
PollRequestTypeUnknown: QueuesDownstreamRequestType
Get: QueuesDownstreamRequestType
AckAll: QueuesDownstreamRequestType
AckRange: QueuesDownstreamRequestType
NAckAll: QueuesDownstreamRequestType
NAckRange: QueuesDownstreamRequestType
ReQueueAll: QueuesDownstreamRequestType
ReQueueRange: QueuesDownstreamRequestType
ActiveOffsets: QueuesDownstreamRequestType
TransactionStatus: QueuesDownstreamRequestType
CloseByClient: QueuesDownstreamRequestType
CloseByServer: QueuesDownstreamRequestType

class PingResult(_message.Message):
    __slots__ = ("Host", "Version", "ServerStartTime", "ServerUpTimeSeconds")
    HOST_FIELD_NUMBER: _ClassVar[int]
    VERSION_FIELD_NUMBER: _ClassVar[int]
    SERVERSTARTTIME_FIELD_NUMBER: _ClassVar[int]
    SERVERUPTIMESECONDS_FIELD_NUMBER: _ClassVar[int]
    Host: str
    Version: str
    ServerStartTime: int
    ServerUpTimeSeconds: int
    def __init__(
        self,
        Host: _Optional[str] = ...,
        Version: _Optional[str] = ...,
        ServerStartTime: _Optional[int] = ...,
        ServerUpTimeSeconds: _Optional[int] = ...,
    ) -> None: ...

class Empty(_message.Message):
    __slots__ = ()
    def __init__(self) -> None: ...

class Result(_message.Message):
    __slots__ = ("EventID", "Sent", "Error")
    EVENTID_FIELD_NUMBER: _ClassVar[int]
    SENT_FIELD_NUMBER: _ClassVar[int]
    ERROR_FIELD_NUMBER: _ClassVar[int]
    EventID: str
    Sent: bool
    Error: str
    def __init__(
        self,
        EventID: _Optional[str] = ...,
        Sent: bool = ...,
        Error: _Optional[str] = ...,
    ) -> None: ...

class Event(_message.Message):
    __slots__ = ("EventID", "ClientID", "Channel", "Metadata", "Body", "Store", "Tags")
    class TagsEntry(_message.Message):
        __slots__ = ("key", "value")
        KEY_FIELD_NUMBER: _ClassVar[int]
        VALUE_FIELD_NUMBER: _ClassVar[int]
        key: str
        value: str
        def __init__(
            self, key: _Optional[str] = ..., value: _Optional[str] = ...
        ) -> None: ...

    EVENTID_FIELD_NUMBER: _ClassVar[int]
    CLIENTID_FIELD_NUMBER: _ClassVar[int]
    CHANNEL_FIELD_NUMBER: _ClassVar[int]
    METADATA_FIELD_NUMBER: _ClassVar[int]
    BODY_FIELD_NUMBER: _ClassVar[int]
    STORE_FIELD_NUMBER: _ClassVar[int]
    TAGS_FIELD_NUMBER: _ClassVar[int]
    EventID: str
    ClientID: str
    Channel: str
    Metadata: str
    Body: bytes
    Store: bool
    Tags: _containers.ScalarMap[str, str]
    def __init__(
        self,
        EventID: _Optional[str] = ...,
        ClientID: _Optional[str] = ...,
        Channel: _Optional[str] = ...,
        Metadata: _Optional[str] = ...,
        Body: _Optional[bytes] = ...,
        Store: bool = ...,
        Tags: _Optional[_Mapping[str, str]] = ...,
    ) -> None: ...

class EventReceive(_message.Message):
    __slots__ = (
        "EventID",
        "Channel",
        "Metadata",
        "Body",
        "Timestamp",
        "Sequence",
        "Tags",
    )
    class TagsEntry(_message.Message):
        __slots__ = ("key", "value")
        KEY_FIELD_NUMBER: _ClassVar[int]
        VALUE_FIELD_NUMBER: _ClassVar[int]
        key: str
        value: str
        def __init__(
            self, key: _Optional[str] = ..., value: _Optional[str] = ...
        ) -> None: ...

    EVENTID_FIELD_NUMBER: _ClassVar[int]
    CHANNEL_FIELD_NUMBER: _ClassVar[int]
    METADATA_FIELD_NUMBER: _ClassVar[int]
    BODY_FIELD_NUMBER: _ClassVar[int]
    TIMESTAMP_FIELD_NUMBER: _ClassVar[int]
    SEQUENCE_FIELD_NUMBER: _ClassVar[int]
    TAGS_FIELD_NUMBER: _ClassVar[int]
    EventID: str
    Channel: str
    Metadata: str
    Body: bytes
    Timestamp: int
    Sequence: int
    Tags: _containers.ScalarMap[str, str]
    def __init__(
        self,
        EventID: _Optional[str] = ...,
        Channel: _Optional[str] = ...,
        Metadata: _Optional[str] = ...,
        Body: _Optional[bytes] = ...,
        Timestamp: _Optional[int] = ...,
        Sequence: _Optional[int] = ...,
        Tags: _Optional[_Mapping[str, str]] = ...,
    ) -> None: ...

class Subscribe(_message.Message):
    __slots__ = (
        "SubscribeTypeData",
        "ClientID",
        "Channel",
        "Group",
        "EventsStoreTypeData",
        "EventsStoreTypeValue",
    )
    class SubscribeType(int, metaclass=_enum_type_wrapper.EnumTypeWrapper):
        __slots__ = ()
        SubscribeTypeUndefined: _ClassVar[Subscribe.SubscribeType]
        Events: _ClassVar[Subscribe.SubscribeType]
        EventsStore: _ClassVar[Subscribe.SubscribeType]
        Commands: _ClassVar[Subscribe.SubscribeType]
        Queries: _ClassVar[Subscribe.SubscribeType]

    SubscribeTypeUndefined: Subscribe.SubscribeType
    Events: Subscribe.SubscribeType
    EventsStore: Subscribe.SubscribeType
    Commands: Subscribe.SubscribeType
    Queries: Subscribe.SubscribeType
    class EventsStoreType(int, metaclass=_enum_type_wrapper.EnumTypeWrapper):
        __slots__ = ()
        EventsStoreTypeUndefined: _ClassVar[Subscribe.EventsStoreType]
        StartNewOnly: _ClassVar[Subscribe.EventsStoreType]
        StartFromFirst: _ClassVar[Subscribe.EventsStoreType]
        StartFromLast: _ClassVar[Subscribe.EventsStoreType]
        StartAtSequence: _ClassVar[Subscribe.EventsStoreType]
        StartAtTime: _ClassVar[Subscribe.EventsStoreType]
        StartAtTimeDelta: _ClassVar[Subscribe.EventsStoreType]

    EventsStoreTypeUndefined: Subscribe.EventsStoreType
    StartNewOnly: Subscribe.EventsStoreType
    StartFromFirst: Subscribe.EventsStoreType
    StartFromLast: Subscribe.EventsStoreType
    StartAtSequence: Subscribe.EventsStoreType
    StartAtTime: Subscribe.EventsStoreType
    StartAtTimeDelta: Subscribe.EventsStoreType
    SUBSCRIBETYPEDATA_FIELD_NUMBER: _ClassVar[int]
    CLIENTID_FIELD_NUMBER: _ClassVar[int]
    CHANNEL_FIELD_NUMBER: _ClassVar[int]
    GROUP_FIELD_NUMBER: _ClassVar[int]
    EVENTSSTORETYPEDATA_FIELD_NUMBER: _ClassVar[int]
    EVENTSSTORETYPEVALUE_FIELD_NUMBER: _ClassVar[int]
    SubscribeTypeData: Subscribe.SubscribeType
    ClientID: str
    Channel: str
    Group: str
    EventsStoreTypeData: Subscribe.EventsStoreType
    EventsStoreTypeValue: int
    def __init__(
        self,
        SubscribeTypeData: _Optional[_Union[Subscribe.SubscribeType, str]] = ...,
        ClientID: _Optional[str] = ...,
        Channel: _Optional[str] = ...,
        Group: _Optional[str] = ...,
        EventsStoreTypeData: _Optional[_Union[Subscribe.EventsStoreType, str]] = ...,
        EventsStoreTypeValue: _Optional[int] = ...,
    ) -> None: ...

class Request(_message.Message):
    __slots__ = (
        "RequestID",
        "RequestTypeData",
        "ClientID",
        "Channel",
        "Metadata",
        "Body",
        "ReplyChannel",
        "Timeout",
        "CacheKey",
        "CacheTTL",
        "Span",
        "Tags",
    )
    class RequestType(int, metaclass=_enum_type_wrapper.EnumTypeWrapper):
        __slots__ = ()
        RequestTypeUnknown: _ClassVar[Request.RequestType]
        Command: _ClassVar[Request.RequestType]
        Query: _ClassVar[Request.RequestType]

    RequestTypeUnknown: Request.RequestType
    Command: Request.RequestType
    Query: Request.RequestType
    class TagsEntry(_message.Message):
        __slots__ = ("key", "value")
        KEY_FIELD_NUMBER: _ClassVar[int]
        VALUE_FIELD_NUMBER: _ClassVar[int]
        key: str
        value: str
        def __init__(
            self, key: _Optional[str] = ..., value: _Optional[str] = ...
        ) -> None: ...

    REQUESTID_FIELD_NUMBER: _ClassVar[int]
    REQUESTTYPEDATA_FIELD_NUMBER: _ClassVar[int]
    CLIENTID_FIELD_NUMBER: _ClassVar[int]
    CHANNEL_FIELD_NUMBER: _ClassVar[int]
    METADATA_FIELD_NUMBER: _ClassVar[int]
    BODY_FIELD_NUMBER: _ClassVar[int]
    REPLYCHANNEL_FIELD_NUMBER: _ClassVar[int]
    TIMEOUT_FIELD_NUMBER: _ClassVar[int]
    CACHEKEY_FIELD_NUMBER: _ClassVar[int]
    CACHETTL_FIELD_NUMBER: _ClassVar[int]
    SPAN_FIELD_NUMBER: _ClassVar[int]
    TAGS_FIELD_NUMBER: _ClassVar[int]
    RequestID: str
    RequestTypeData: Request.RequestType
    ClientID: str
    Channel: str
    Metadata: str
    Body: bytes
    ReplyChannel: str
    Timeout: int
    CacheKey: str
    CacheTTL: int
    Span: bytes
    Tags: _containers.ScalarMap[str, str]
    def __init__(
        self,
        RequestID: _Optional[str] = ...,
        RequestTypeData: _Optional[_Union[Request.RequestType, str]] = ...,
        ClientID: _Optional[str] = ...,
        Channel: _Optional[str] = ...,
        Metadata: _Optional[str] = ...,
        Body: _Optional[bytes] = ...,
        ReplyChannel: _Optional[str] = ...,
        Timeout: _Optional[int] = ...,
        CacheKey: _Optional[str] = ...,
        CacheTTL: _Optional[int] = ...,
        Span: _Optional[bytes] = ...,
        Tags: _Optional[_Mapping[str, str]] = ...,
    ) -> None: ...

class Response(_message.Message):
    __slots__ = (
        "ClientID",
        "RequestID",
        "ReplyChannel",
        "Metadata",
        "Body",
        "CacheHit",
        "Timestamp",
        "Executed",
        "Error",
        "Span",
        "Tags",
    )
    class TagsEntry(_message.Message):
        __slots__ = ("key", "value")
        KEY_FIELD_NUMBER: _ClassVar[int]
        VALUE_FIELD_NUMBER: _ClassVar[int]
        key: str
        value: str
        def __init__(
            self, key: _Optional[str] = ..., value: _Optional[str] = ...
        ) -> None: ...

    CLIENTID_FIELD_NUMBER: _ClassVar[int]
    REQUESTID_FIELD_NUMBER: _ClassVar[int]
    REPLYCHANNEL_FIELD_NUMBER: _ClassVar[int]
    METADATA_FIELD_NUMBER: _ClassVar[int]
    BODY_FIELD_NUMBER: _ClassVar[int]
    CACHEHIT_FIELD_NUMBER: _ClassVar[int]
    TIMESTAMP_FIELD_NUMBER: _ClassVar[int]
    EXECUTED_FIELD_NUMBER: _ClassVar[int]
    ERROR_FIELD_NUMBER: _ClassVar[int]
    SPAN_FIELD_NUMBER: _ClassVar[int]
    TAGS_FIELD_NUMBER: _ClassVar[int]
    ClientID: str
    RequestID: str
    ReplyChannel: str
    Metadata: str
    Body: bytes
    CacheHit: bool
    Timestamp: int
    Executed: bool
    Error: str
    Span: bytes
    Tags: _containers.ScalarMap[str, str]
    def __init__(
        self,
        ClientID: _Optional[str] = ...,
        RequestID: _Optional[str] = ...,
        ReplyChannel: _Optional[str] = ...,
        Metadata: _Optional[str] = ...,
        Body: _Optional[bytes] = ...,
        CacheHit: bool = ...,
        Timestamp: _Optional[int] = ...,
        Executed: bool = ...,
        Error: _Optional[str] = ...,
        Span: _Optional[bytes] = ...,
        Tags: _Optional[_Mapping[str, str]] = ...,
    ) -> None: ...

class QueueMessage(_message.Message):
    __slots__ = (
        "MessageID",
        "ClientID",
        "Channel",
        "Metadata",
        "Body",
        "Tags",
        "Attributes",
        "Policy",
        "Topic",
        "Partition",
        "PartitionKey",
    )
    class TagsEntry(_message.Message):
        __slots__ = ("key", "value")
        KEY_FIELD_NUMBER: _ClassVar[int]
        VALUE_FIELD_NUMBER: _ClassVar[int]
        key: str
        value: str
        def __init__(
            self, key: _Optional[str] = ..., value: _Optional[str] = ...
        ) -> None: ...

    MESSAGEID_FIELD_NUMBER: _ClassVar[int]
    CLIENTID_FIELD_NUMBER: _ClassVar[int]
    CHANNEL_FIELD_NUMBER: _ClassVar[int]
    METADATA_FIELD_NUMBER: _ClassVar[int]
    BODY_FIELD_NUMBER: _ClassVar[int]
    TAGS_FIELD_NUMBER: _ClassVar[int]
    ATTRIBUTES_FIELD_NUMBER: _ClassVar[int]
    POLICY_FIELD_NUMBER: _ClassVar[int]
    TOPIC_FIELD_NUMBER: _ClassVar[int]
    PARTITION_FIELD_NUMBER: _ClassVar[int]
    PARTITIONKEY_FIELD_NUMBER: _ClassVar[int]
    MessageID: str
    ClientID: str
    Channel: str
    Metadata: str
    Body: bytes
    Tags: _containers.ScalarMap[str, str]
    Attributes: QueueMessageAttributes
    Policy: QueueMessagePolicy
    Topic: str
    Partition: int
    PartitionKey: str
    def __init__(
        self,
        MessageID: _Optional[str] = ...,
        ClientID: _Optional[str] = ...,
        Channel: _Optional[str] = ...,
        Metadata: _Optional[str] = ...,
        Body: _Optional[bytes] = ...,
        Tags: _Optional[_Mapping[str, str]] = ...,
        Attributes: _Optional[_Union[QueueMessageAttributes, _Mapping]] = ...,
        Policy: _Optional[_Union[QueueMessagePolicy, _Mapping]] = ...,
        Topic: _Optional[str] = ...,
        Partition: _Optional[int] = ...,
        PartitionKey: _Optional[str] = ...,
    ) -> None: ...

class QueueMessagesBatchRequest(_message.Message):
    __slots__ = ("BatchID", "Messages")
    BATCHID_FIELD_NUMBER: _ClassVar[int]
    MESSAGES_FIELD_NUMBER: _ClassVar[int]
    BatchID: str
    Messages: _containers.RepeatedCompositeFieldContainer[QueueMessage]
    def __init__(
        self,
        BatchID: _Optional[str] = ...,
        Messages: _Optional[_Iterable[_Union[QueueMessage, _Mapping]]] = ...,
    ) -> None: ...

class QueueMessagesBatchResponse(_message.Message):
    __slots__ = ("BatchID", "Results", "HaveErrors")
    BATCHID_FIELD_NUMBER: _ClassVar[int]
    RESULTS_FIELD_NUMBER: _ClassVar[int]
    HAVEERRORS_FIELD_NUMBER: _ClassVar[int]
    BatchID: str
    Results: _containers.RepeatedCompositeFieldContainer[SendQueueMessageResult]
    HaveErrors: bool
    def __init__(
        self,
        BatchID: _Optional[str] = ...,
        Results: _Optional[_Iterable[_Union[SendQueueMessageResult, _Mapping]]] = ...,
        HaveErrors: bool = ...,
    ) -> None: ...

class QueueMessageAttributes(_message.Message):
    __slots__ = (
        "Timestamp",
        "Sequence",
        "MD5OfBody",
        "ReceiveCount",
        "ReRouted",
        "ReRoutedFromQueue",
        "ExpirationAt",
        "DelayedTo",
    )
    TIMESTAMP_FIELD_NUMBER: _ClassVar[int]
    SEQUENCE_FIELD_NUMBER: _ClassVar[int]
    MD5OFBODY_FIELD_NUMBER: _ClassVar[int]
    RECEIVECOUNT_FIELD_NUMBER: _ClassVar[int]
    REROUTED_FIELD_NUMBER: _ClassVar[int]
    REROUTEDFROMQUEUE_FIELD_NUMBER: _ClassVar[int]
    EXPIRATIONAT_FIELD_NUMBER: _ClassVar[int]
    DELAYEDTO_FIELD_NUMBER: _ClassVar[int]
    Timestamp: int
    Sequence: int
    MD5OfBody: str
    ReceiveCount: int
    ReRouted: bool
    ReRoutedFromQueue: str
    ExpirationAt: int
    DelayedTo: int
    def __init__(
        self,
        Timestamp: _Optional[int] = ...,
        Sequence: _Optional[int] = ...,
        MD5OfBody: _Optional[str] = ...,
        ReceiveCount: _Optional[int] = ...,
        ReRouted: bool = ...,
        ReRoutedFromQueue: _Optional[str] = ...,
        ExpirationAt: _Optional[int] = ...,
        DelayedTo: _Optional[int] = ...,
    ) -> None: ...

class QueueMessagePolicy(_message.Message):
    __slots__ = (
        "ExpirationSeconds",
        "DelaySeconds",
        "MaxReceiveCount",
        "MaxReceiveQueue",
    )
    EXPIRATIONSECONDS_FIELD_NUMBER: _ClassVar[int]
    DELAYSECONDS_FIELD_NUMBER: _ClassVar[int]
    MAXRECEIVECOUNT_FIELD_NUMBER: _ClassVar[int]
    MAXRECEIVEQUEUE_FIELD_NUMBER: _ClassVar[int]
    ExpirationSeconds: int
    DelaySeconds: int
    MaxReceiveCount: int
    MaxReceiveQueue: str
    def __init__(
        self,
        ExpirationSeconds: _Optional[int] = ...,
        DelaySeconds: _Optional[int] = ...,
        MaxReceiveCount: _Optional[int] = ...,
        MaxReceiveQueue: _Optional[str] = ...,
    ) -> None: ...

class SendQueueMessageResult(_message.Message):
    __slots__ = (
        "MessageID",
        "SentAt",
        "ExpirationAt",
        "DelayedTo",
        "IsError",
        "Error",
        "RefChannel",
        "RefTopic",
        "RefPartition",
        "RefHash",
    )
    MESSAGEID_FIELD_NUMBER: _ClassVar[int]
    SENTAT_FIELD_NUMBER: _ClassVar[int]
    EXPIRATIONAT_FIELD_NUMBER: _ClassVar[int]
    DELAYEDTO_FIELD_NUMBER: _ClassVar[int]
    ISERROR_FIELD_NUMBER: _ClassVar[int]
    ERROR_FIELD_NUMBER: _ClassVar[int]
    REFCHANNEL_FIELD_NUMBER: _ClassVar[int]
    REFTOPIC_FIELD_NUMBER: _ClassVar[int]
    REFPARTITION_FIELD_NUMBER: _ClassVar[int]
    REFHASH_FIELD_NUMBER: _ClassVar[int]
    MessageID: str
    SentAt: int
    ExpirationAt: int
    DelayedTo: int
    IsError: bool
    Error: str
    RefChannel: str
    RefTopic: str
    RefPartition: int
    RefHash: str
    def __init__(
        self,
        MessageID: _Optional[str] = ...,
        SentAt: _Optional[int] = ...,
        ExpirationAt: _Optional[int] = ...,
        DelayedTo: _Optional[int] = ...,
        IsError: bool = ...,
        Error: _Optional[str] = ...,
        RefChannel: _Optional[str] = ...,
        RefTopic: _Optional[str] = ...,
        RefPartition: _Optional[int] = ...,
        RefHash: _Optional[str] = ...,
    ) -> None: ...

class ReceiveQueueMessagesRequest(_message.Message):
    __slots__ = (
        "RequestID",
        "ClientID",
        "Channel",
        "MaxNumberOfMessages",
        "WaitTimeSeconds",
        "IsPeak",
    )
    REQUESTID_FIELD_NUMBER: _ClassVar[int]
    CLIENTID_FIELD_NUMBER: _ClassVar[int]
    CHANNEL_FIELD_NUMBER: _ClassVar[int]
    MAXNUMBEROFMESSAGES_FIELD_NUMBER: _ClassVar[int]
    WAITTIMESECONDS_FIELD_NUMBER: _ClassVar[int]
    ISPEAK_FIELD_NUMBER: _ClassVar[int]
    RequestID: str
    ClientID: str
    Channel: str
    MaxNumberOfMessages: int
    WaitTimeSeconds: int
    IsPeak: bool
    def __init__(
        self,
        RequestID: _Optional[str] = ...,
        ClientID: _Optional[str] = ...,
        Channel: _Optional[str] = ...,
        MaxNumberOfMessages: _Optional[int] = ...,
        WaitTimeSeconds: _Optional[int] = ...,
        IsPeak: bool = ...,
    ) -> None: ...

class ReceiveQueueMessagesResponse(_message.Message):
    __slots__ = (
        "RequestID",
        "Messages",
        "MessagesReceived",
        "MessagesExpired",
        "IsPeak",
        "IsError",
        "Error",
    )
    REQUESTID_FIELD_NUMBER: _ClassVar[int]
    MESSAGES_FIELD_NUMBER: _ClassVar[int]
    MESSAGESRECEIVED_FIELD_NUMBER: _ClassVar[int]
    MESSAGESEXPIRED_FIELD_NUMBER: _ClassVar[int]
    ISPEAK_FIELD_NUMBER: _ClassVar[int]
    ISERROR_FIELD_NUMBER: _ClassVar[int]
    ERROR_FIELD_NUMBER: _ClassVar[int]
    RequestID: str
    Messages: _containers.RepeatedCompositeFieldContainer[QueueMessage]
    MessagesReceived: int
    MessagesExpired: int
    IsPeak: bool
    IsError: bool
    Error: str
    def __init__(
        self,
        RequestID: _Optional[str] = ...,
        Messages: _Optional[_Iterable[_Union[QueueMessage, _Mapping]]] = ...,
        MessagesReceived: _Optional[int] = ...,
        MessagesExpired: _Optional[int] = ...,
        IsPeak: bool = ...,
        IsError: bool = ...,
        Error: _Optional[str] = ...,
    ) -> None: ...

class AckAllQueueMessagesRequest(_message.Message):
    __slots__ = ("RequestID", "ClientID", "Channel", "WaitTimeSeconds")
    REQUESTID_FIELD_NUMBER: _ClassVar[int]
    CLIENTID_FIELD_NUMBER: _ClassVar[int]
    CHANNEL_FIELD_NUMBER: _ClassVar[int]
    WAITTIMESECONDS_FIELD_NUMBER: _ClassVar[int]
    RequestID: str
    ClientID: str
    Channel: str
    WaitTimeSeconds: int
    def __init__(
        self,
        RequestID: _Optional[str] = ...,
        ClientID: _Optional[str] = ...,
        Channel: _Optional[str] = ...,
        WaitTimeSeconds: _Optional[int] = ...,
    ) -> None: ...

class AckAllQueueMessagesResponse(_message.Message):
    __slots__ = ("RequestID", "AffectedMessages", "IsError", "Error")
    REQUESTID_FIELD_NUMBER: _ClassVar[int]
    AFFECTEDMESSAGES_FIELD_NUMBER: _ClassVar[int]
    ISERROR_FIELD_NUMBER: _ClassVar[int]
    ERROR_FIELD_NUMBER: _ClassVar[int]
    RequestID: str
    AffectedMessages: int
    IsError: bool
    Error: str
    def __init__(
        self,
        RequestID: _Optional[str] = ...,
        AffectedMessages: _Optional[int] = ...,
        IsError: bool = ...,
        Error: _Optional[str] = ...,
    ) -> None: ...

class StreamQueueMessagesRequest(_message.Message):
    __slots__ = (
        "RequestID",
        "ClientID",
        "StreamRequestTypeData",
        "Channel",
        "VisibilitySeconds",
        "WaitTimeSeconds",
        "RefSequence",
        "ModifiedMessage",
    )
    REQUESTID_FIELD_NUMBER: _ClassVar[int]
    CLIENTID_FIELD_NUMBER: _ClassVar[int]
    STREAMREQUESTTYPEDATA_FIELD_NUMBER: _ClassVar[int]
    CHANNEL_FIELD_NUMBER: _ClassVar[int]
    VISIBILITYSECONDS_FIELD_NUMBER: _ClassVar[int]
    WAITTIMESECONDS_FIELD_NUMBER: _ClassVar[int]
    REFSEQUENCE_FIELD_NUMBER: _ClassVar[int]
    MODIFIEDMESSAGE_FIELD_NUMBER: _ClassVar[int]
    RequestID: str
    ClientID: str
    StreamRequestTypeData: StreamRequestType
    Channel: str
    VisibilitySeconds: int
    WaitTimeSeconds: int
    RefSequence: int
    ModifiedMessage: QueueMessage
    def __init__(
        self,
        RequestID: _Optional[str] = ...,
        ClientID: _Optional[str] = ...,
        StreamRequestTypeData: _Optional[_Union[StreamRequestType, str]] = ...,
        Channel: _Optional[str] = ...,
        VisibilitySeconds: _Optional[int] = ...,
        WaitTimeSeconds: _Optional[int] = ...,
        RefSequence: _Optional[int] = ...,
        ModifiedMessage: _Optional[_Union[QueueMessage, _Mapping]] = ...,
    ) -> None: ...

class StreamQueueMessagesResponse(_message.Message):
    __slots__ = ("RequestID", "StreamRequestTypeData", "Message", "IsError", "Error")
    REQUESTID_FIELD_NUMBER: _ClassVar[int]
    STREAMREQUESTTYPEDATA_FIELD_NUMBER: _ClassVar[int]
    MESSAGE_FIELD_NUMBER: _ClassVar[int]
    ISERROR_FIELD_NUMBER: _ClassVar[int]
    ERROR_FIELD_NUMBER: _ClassVar[int]
    RequestID: str
    StreamRequestTypeData: StreamRequestType
    Message: QueueMessage
    IsError: bool
    Error: str
    def __init__(
        self,
        RequestID: _Optional[str] = ...,
        StreamRequestTypeData: _Optional[_Union[StreamRequestType, str]] = ...,
        Message: _Optional[_Union[QueueMessage, _Mapping]] = ...,
        IsError: bool = ...,
        Error: _Optional[str] = ...,
    ) -> None: ...

class QueuesUpstreamRequest(_message.Message):
    __slots__ = ("RequestID", "Messages")
    REQUESTID_FIELD_NUMBER: _ClassVar[int]
    MESSAGES_FIELD_NUMBER: _ClassVar[int]
    RequestID: str
    Messages: _containers.RepeatedCompositeFieldContainer[QueueMessage]
    def __init__(
        self,
        RequestID: _Optional[str] = ...,
        Messages: _Optional[_Iterable[_Union[QueueMessage, _Mapping]]] = ...,
    ) -> None: ...

class QueuesUpstreamResponse(_message.Message):
    __slots__ = ("RefRequestID", "Results", "IsError", "Error")
    REFREQUESTID_FIELD_NUMBER: _ClassVar[int]
    RESULTS_FIELD_NUMBER: _ClassVar[int]
    ISERROR_FIELD_NUMBER: _ClassVar[int]
    ERROR_FIELD_NUMBER: _ClassVar[int]
    RefRequestID: str
    Results: _containers.RepeatedCompositeFieldContainer[SendQueueMessageResult]
    IsError: bool
    Error: str
    def __init__(
        self,
        RefRequestID: _Optional[str] = ...,
        Results: _Optional[_Iterable[_Union[SendQueueMessageResult, _Mapping]]] = ...,
        IsError: bool = ...,
        Error: _Optional[str] = ...,
    ) -> None: ...

class QueuesDownstreamRequest(_message.Message):
    __slots__ = (
        "RequestID",
        "ClientID",
        "RequestTypeData",
        "Channel",
        "MaxItems",
        "WaitTimeout",
        "AutoAck",
        "ReQueueChannel",
        "SequenceRange",
        "RefTransactionId",
        "Metadata",
    )
    class MetadataEntry(_message.Message):
        __slots__ = ("key", "value")
        KEY_FIELD_NUMBER: _ClassVar[int]
        VALUE_FIELD_NUMBER: _ClassVar[int]
        key: str
        value: str
        def __init__(
            self, key: _Optional[str] = ..., value: _Optional[str] = ...
        ) -> None: ...

    REQUESTID_FIELD_NUMBER: _ClassVar[int]
    CLIENTID_FIELD_NUMBER: _ClassVar[int]
    REQUESTTYPEDATA_FIELD_NUMBER: _ClassVar[int]
    CHANNEL_FIELD_NUMBER: _ClassVar[int]
    MAXITEMS_FIELD_NUMBER: _ClassVar[int]
    WAITTIMEOUT_FIELD_NUMBER: _ClassVar[int]
    AUTOACK_FIELD_NUMBER: _ClassVar[int]
    REQUEUECHANNEL_FIELD_NUMBER: _ClassVar[int]
    SEQUENCERANGE_FIELD_NUMBER: _ClassVar[int]
    REFTRANSACTIONID_FIELD_NUMBER: _ClassVar[int]
    METADATA_FIELD_NUMBER: _ClassVar[int]
    RequestID: str
    ClientID: str
    RequestTypeData: QueuesDownstreamRequestType
    Channel: str
    MaxItems: int
    WaitTimeout: int
    AutoAck: bool
    ReQueueChannel: str
    SequenceRange: _containers.RepeatedScalarFieldContainer[int]
    RefTransactionId: str
    Metadata: _containers.ScalarMap[str, str]
    def __init__(
        self,
        RequestID: _Optional[str] = ...,
        ClientID: _Optional[str] = ...,
        RequestTypeData: _Optional[_Union[QueuesDownstreamRequestType, str]] = ...,
        Channel: _Optional[str] = ...,
        MaxItems: _Optional[int] = ...,
        WaitTimeout: _Optional[int] = ...,
        AutoAck: bool = ...,
        ReQueueChannel: _Optional[str] = ...,
        SequenceRange: _Optional[_Iterable[int]] = ...,
        RefTransactionId: _Optional[str] = ...,
        Metadata: _Optional[_Mapping[str, str]] = ...,
    ) -> None: ...

class QueuesDownstreamResponse(_message.Message):
    __slots__ = (
        "TransactionId",
        "RefRequestId",
        "RequestTypeData",
        "Messages",
        "ActiveOffsets",
        "IsError",
        "Error",
        "TransactionComplete",
        "Metadata",
    )
    class MetadataEntry(_message.Message):
        __slots__ = ("key", "value")
        KEY_FIELD_NUMBER: _ClassVar[int]
        VALUE_FIELD_NUMBER: _ClassVar[int]
        key: str
        value: str
        def __init__(
            self, key: _Optional[str] = ..., value: _Optional[str] = ...
        ) -> None: ...

    TRANSACTIONID_FIELD_NUMBER: _ClassVar[int]
    REFREQUESTID_FIELD_NUMBER: _ClassVar[int]
    REQUESTTYPEDATA_FIELD_NUMBER: _ClassVar[int]
    MESSAGES_FIELD_NUMBER: _ClassVar[int]
    ACTIVEOFFSETS_FIELD_NUMBER: _ClassVar[int]
    ISERROR_FIELD_NUMBER: _ClassVar[int]
    ERROR_FIELD_NUMBER: _ClassVar[int]
    TRANSACTIONCOMPLETE_FIELD_NUMBER: _ClassVar[int]
    METADATA_FIELD_NUMBER: _ClassVar[int]
    TransactionId: str
    RefRequestId: str
    RequestTypeData: QueuesDownstreamRequestType
    Messages: _containers.RepeatedCompositeFieldContainer[QueueMessage]
    ActiveOffsets: _containers.RepeatedScalarFieldContainer[int]
    IsError: bool
    Error: str
    TransactionComplete: bool
    Metadata: _containers.ScalarMap[str, str]
    def __init__(
        self,
        TransactionId: _Optional[str] = ...,
        RefRequestId: _Optional[str] = ...,
        RequestTypeData: _Optional[_Union[QueuesDownstreamRequestType, str]] = ...,
        Messages: _Optional[_Iterable[_Union[QueueMessage, _Mapping]]] = ...,
        ActiveOffsets: _Optional[_Iterable[int]] = ...,
        IsError: bool = ...,
        Error: _Optional[str] = ...,
        TransactionComplete: bool = ...,
        Metadata: _Optional[_Mapping[str, str]] = ...,
    ) -> None: ...

class QueueInfo(_message.Message):
    __slots__ = (
        "Name",
        "Messages",
        "Bytes",
        "FirstSequence",
        "LastSequence",
        "Sent",
        "Delivered",
        "Waiting",
        "Subscribers",
    )
    NAME_FIELD_NUMBER: _ClassVar[int]
    MESSAGES_FIELD_NUMBER: _ClassVar[int]
    BYTES_FIELD_NUMBER: _ClassVar[int]
    FIRSTSEQUENCE_FIELD_NUMBER: _ClassVar[int]
    LASTSEQUENCE_FIELD_NUMBER: _ClassVar[int]
    SENT_FIELD_NUMBER: _ClassVar[int]
    DELIVERED_FIELD_NUMBER: _ClassVar[int]
    WAITING_FIELD_NUMBER: _ClassVar[int]
    SUBSCRIBERS_FIELD_NUMBER: _ClassVar[int]
    Name: str
    Messages: int
    Bytes: int
    FirstSequence: int
    LastSequence: int
    Sent: int
    Delivered: int
    Waiting: int
    Subscribers: int
    def __init__(
        self,
        Name: _Optional[str] = ...,
        Messages: _Optional[int] = ...,
        Bytes: _Optional[int] = ...,
        FirstSequence: _Optional[int] = ...,
        LastSequence: _Optional[int] = ...,
        Sent: _Optional[int] = ...,
        Delivered: _Optional[int] = ...,
        Waiting: _Optional[int] = ...,
        Subscribers: _Optional[int] = ...,
    ) -> None: ...

class QueuesInfo(_message.Message):
    __slots__ = ("TotalQueue", "Sent", "Delivered", "Waiting", "Queues")
    TOTALQUEUE_FIELD_NUMBER: _ClassVar[int]
    SENT_FIELD_NUMBER: _ClassVar[int]
    DELIVERED_FIELD_NUMBER: _ClassVar[int]
    WAITING_FIELD_NUMBER: _ClassVar[int]
    QUEUES_FIELD_NUMBER: _ClassVar[int]
    TotalQueue: int
    Sent: int
    Delivered: int
    Waiting: int
    Queues: _containers.RepeatedCompositeFieldContainer[QueueInfo]
    def __init__(
        self,
        TotalQueue: _Optional[int] = ...,
        Sent: _Optional[int] = ...,
        Delivered: _Optional[int] = ...,
        Waiting: _Optional[int] = ...,
        Queues: _Optional[_Iterable[_Union[QueueInfo, _Mapping]]] = ...,
    ) -> None: ...

class QueuesInfoRequest(_message.Message):
    __slots__ = ("RequestID", "QueueName")
    REQUESTID_FIELD_NUMBER: _ClassVar[int]
    QUEUENAME_FIELD_NUMBER: _ClassVar[int]
    RequestID: str
    QueueName: str
    def __init__(
        self, RequestID: _Optional[str] = ..., QueueName: _Optional[str] = ...
    ) -> None: ...

class QueuesInfoResponse(_message.Message):
    __slots__ = ("RefRequestID", "Info")
    REFREQUESTID_FIELD_NUMBER: _ClassVar[int]
    INFO_FIELD_NUMBER: _ClassVar[int]
    RefRequestID: str
    Info: QueuesInfo
    def __init__(
        self,
        RefRequestID: _Optional[str] = ...,
        Info: _Optional[_Union[QueuesInfo, _Mapping]] = ...,
    ) -> None: ...



from datetime import datetime
from typing import Dict, Optional
from pydantic import BaseModel, Field, field_validator
from kubemq.cq.query_message_received import QueryMessageReceived
from kubemq.grpc import Response as pbResponse


class QueryResponseMessage(BaseModel):
    """
    Class for representing a query response message.

    Attributes:
        query_received (Optional[QueryMessageReceived]): The received query message.
        client_id (str): The client ID.
        request_id (str): The request ID.
        is_executed (bool): Indicates if the query has been executed.
        timestamp (datetime): The timestamp of the query response.
        error (str): The error message, if any.
        metadata (Optional[str]): The metadata associated with the query response.
        body (bytes): The body of the query response.
        tags (Dict[str, str]): The tags associated with the query response.
    """

    query_received: Optional[QueryMessageReceived] = None
    client_id: str = Field(default="")
    request_id: str = Field(default="")
    is_executed: bool = Field(default=False)
    timestamp: datetime = Field(default_factory=datetime.now)
    error: str = Field(default="")
    metadata: Optional[str] = None
    body: bytes = Field(default=b"")
    tags: Dict[str, str] = Field(default_factory=dict)

    model_config = {"arbitrary_types_allowed": True}

    @field_validator("query_received")
    def validate_query_received(cls, v):
        if v is None:
            raise ValueError("Query response must have a query request.")
        if v.reply_channel == "":
            raise ValueError("Query response must have a reply channel.")
        return v

    @classmethod
    def decode(cls, pb_response: pbResponse) -> "QueryResponseMessage":
        """
        Decodes the protocol buffer response and creates a new QueryResponseMessage instance.
        """
        return cls(
            client_id=pb_response.ClientID,
            request_id=pb_response.RequestID,
            is_executed=pb_response.Executed,
            error=pb_response.Error,
            timestamp=datetime.fromtimestamp(pb_response.Timestamp / 1e9),
            metadata=pb_response.Metadata,
            body=pb_response.Body,
            tags=dict(pb_response.Tags),
        )

    def encode(self, client_id: str) -> pbResponse:
        """
        Encodes the query response message into a protocol buffer response.
        """
        if not self.query_received:
            raise ValueError("Query received is required for encoding.")
        pb_response = pbResponse()
        pb_response.ClientID = client_id
        pb_response.RequestID = self.query_received.id
        pb_response.ReplyChannel = self.query_received.reply_channel
        pb_response.Executed = self.is_executed
        pb_response.Error = self.error
        pb_response.Timestamp = int(self.timestamp.timestamp() * 1e9)
        pb_response.Metadata = self.metadata or ""
        pb_response.Body = self.body or b""
        for key, value in self.tags.items():
            pb_response.Tags[key] = value
        return pb_response

    def __repr__(self) -> str:
        return (
            f"QueryResponseMessage: client_id={self.client_id}, "
            f"request_id={self.request_id}, is_executed={self.is_executed}, "
            f"error={self.error}, timestamp={self.timestamp}"
        )

    @classmethod
    def create(
        cls,
        query_received: Optional[QueryMessageReceived] = None,
        metadata: Optional[str] = None,
        body: bytes = b"",
        tags: Optional[Dict[str, str]] = None,
        is_executed: bool = False,
        error: str = "",
        timestamp: Optional[datetime] = None,
    ) -> "QueryResponseMessage":
        """
        Creates a new QueryResponseMessage instance.

        This method provides backwards compatibility with the original constructor.
        """
        return cls(
            query_received=query_received,
            metadata=metadata,
            body=body,
            tags=tags,
            is_executed=is_executed,
            error=error,
            timestamp=timestamp or datetime.now(),
        )



# Generated by the gRPC Python protocol compiler plugin. DO NOT EDIT!
"""Client and server classes corresponding to protobuf-defined services."""

import grpc

from kubemq.grpc import kubemq_pb2 as kubemq_dot_grpc_dot_kubemq__pb2

GRPC_GENERATED_VERSION = "1.71.0"
GRPC_VERSION = grpc.__version__
_version_not_supported = False

try:
    from grpc._utilities import first_version_is_lower

    _version_not_supported = first_version_is_lower(
        GRPC_VERSION, GRPC_GENERATED_VERSION
    )
except ImportError:
    _version_not_supported = True

if _version_not_supported:
    raise RuntimeError(
        f"The grpc package installed is at version {GRPC_VERSION},"
        + " but the generated code in kubemq/grpc/kubemq_pb2_grpc.py depends on"
        + f" grpcio>={GRPC_GENERATED_VERSION}."
        + f" Please upgrade your grpc module to grpcio>={GRPC_GENERATED_VERSION}"
        + f" or downgrade your generated code using grpcio-tools<={GRPC_VERSION}."
    )


class kubemqStub(object):
    """Missing associated documentation comment in .proto file."""

    def __init__(self, channel):
        """Constructor.

        Args:
            channel: A grpc.Channel.
        """
        self.SendEvent = channel.unary_unary(
            "/kubemq.kubemq/SendEvent",
            request_serializer=kubemq_dot_grpc_dot_kubemq__pb2.Event.SerializeToString,
            response_deserializer=kubemq_dot_grpc_dot_kubemq__pb2.Result.FromString,
            _registered_method=True,
        )
        self.SendEventsStream = channel.stream_stream(
            "/kubemq.kubemq/SendEventsStream",
            request_serializer=kubemq_dot_grpc_dot_kubemq__pb2.Event.SerializeToString,
            response_deserializer=kubemq_dot_grpc_dot_kubemq__pb2.Result.FromString,
            _registered_method=True,
        )
        self.SubscribeToEvents = channel.unary_stream(
            "/kubemq.kubemq/SubscribeToEvents",
            request_serializer=kubemq_dot_grpc_dot_kubemq__pb2.Subscribe.SerializeToString,
            response_deserializer=kubemq_dot_grpc_dot_kubemq__pb2.EventReceive.FromString,
            _registered_method=True,
        )
        self.SubscribeToRequests = channel.unary_stream(
            "/kubemq.kubemq/SubscribeToRequests",
            request_serializer=kubemq_dot_grpc_dot_kubemq__pb2.Subscribe.SerializeToString,
            response_deserializer=kubemq_dot_grpc_dot_kubemq__pb2.Request.FromString,
            _registered_method=True,
        )
        self.SendRequest = channel.unary_unary(
            "/kubemq.kubemq/SendRequest",
            request_serializer=kubemq_dot_grpc_dot_kubemq__pb2.Request.SerializeToString,
            response_deserializer=kubemq_dot_grpc_dot_kubemq__pb2.Response.FromString,
            _registered_method=True,
        )
        self.SendResponse = channel.unary_unary(
            "/kubemq.kubemq/SendResponse",
            request_serializer=kubemq_dot_grpc_dot_kubemq__pb2.Response.SerializeToString,
            response_deserializer=kubemq_dot_grpc_dot_kubemq__pb2.Empty.FromString,
            _registered_method=True,
        )
        self.SendQueueMessage = channel.unary_unary(
            "/kubemq.kubemq/SendQueueMessage",
            request_serializer=kubemq_dot_grpc_dot_kubemq__pb2.QueueMessage.SerializeToString,
            response_deserializer=kubemq_dot_grpc_dot_kubemq__pb2.SendQueueMessageResult.FromString,
            _registered_method=True,
        )
        self.SendQueueMessagesBatch = channel.unary_unary(
            "/kubemq.kubemq/SendQueueMessagesBatch",
            request_serializer=kubemq_dot_grpc_dot_kubemq__pb2.QueueMessagesBatchRequest.SerializeToString,
            response_deserializer=kubemq_dot_grpc_dot_kubemq__pb2.QueueMessagesBatchResponse.FromString,
            _registered_method=True,
        )
        self.ReceiveQueueMessages = channel.unary_unary(
            "/kubemq.kubemq/ReceiveQueueMessages",
            request_serializer=kubemq_dot_grpc_dot_kubemq__pb2.ReceiveQueueMessagesRequest.SerializeToString,
            response_deserializer=kubemq_dot_grpc_dot_kubemq__pb2.ReceiveQueueMessagesResponse.FromString,
            _registered_method=True,
        )
        self.StreamQueueMessage = channel.stream_stream(
            "/kubemq.kubemq/StreamQueueMessage",
            request_serializer=kubemq_dot_grpc_dot_kubemq__pb2.StreamQueueMessagesRequest.SerializeToString,
            response_deserializer=kubemq_dot_grpc_dot_kubemq__pb2.StreamQueueMessagesResponse.FromString,
            _registered_method=True,
        )
        self.AckAllQueueMessages = channel.unary_unary(
            "/kubemq.kubemq/AckAllQueueMessages",
            request_serializer=kubemq_dot_grpc_dot_kubemq__pb2.AckAllQueueMessagesRequest.SerializeToString,
            response_deserializer=kubemq_dot_grpc_dot_kubemq__pb2.AckAllQueueMessagesResponse.FromString,
            _registered_method=True,
        )
        self.Ping = channel.unary_unary(
            "/kubemq.kubemq/Ping",
            request_serializer=kubemq_dot_grpc_dot_kubemq__pb2.Empty.SerializeToString,
            response_deserializer=kubemq_dot_grpc_dot_kubemq__pb2.PingResult.FromString,
            _registered_method=True,
        )
        self.QueuesDownstream = channel.stream_stream(
            "/kubemq.kubemq/QueuesDownstream",
            request_serializer=kubemq_dot_grpc_dot_kubemq__pb2.QueuesDownstreamRequest.SerializeToString,
            response_deserializer=kubemq_dot_grpc_dot_kubemq__pb2.QueuesDownstreamResponse.FromString,
            _registered_method=True,
        )
        self.QueuesUpstream = channel.stream_stream(
            "/kubemq.kubemq/QueuesUpstream",
            request_serializer=kubemq_dot_grpc_dot_kubemq__pb2.QueuesUpstreamRequest.SerializeToString,
            response_deserializer=kubemq_dot_grpc_dot_kubemq__pb2.QueuesUpstreamResponse.FromString,
            _registered_method=True,
        )
        self.QueuesInfo = channel.unary_unary(
            "/kubemq.kubemq/QueuesInfo",
            request_serializer=kubemq_dot_grpc_dot_kubemq__pb2.QueuesInfoRequest.SerializeToString,
            response_deserializer=kubemq_dot_grpc_dot_kubemq__pb2.QueuesInfoResponse.FromString,
            _registered_method=True,
        )


class kubemqServicer(object):
    """Missing associated documentation comment in .proto file."""

    def SendEvent(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details("Method not implemented!")
        raise NotImplementedError("Method not implemented!")

    def SendEventsStream(self, request_iterator, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details("Method not implemented!")
        raise NotImplementedError("Method not implemented!")

    def SubscribeToEvents(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details("Method not implemented!")
        raise NotImplementedError("Method not implemented!")

    def SubscribeToRequests(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details("Method not implemented!")
        raise NotImplementedError("Method not implemented!")

    def SendRequest(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details("Method not implemented!")
        raise NotImplementedError("Method not implemented!")

    def SendResponse(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details("Method not implemented!")
        raise NotImplementedError("Method not implemented!")

    def SendQueueMessage(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details("Method not implemented!")
        raise NotImplementedError("Method not implemented!")

    def SendQueueMessagesBatch(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details("Method not implemented!")
        raise NotImplementedError("Method not implemented!")

    def ReceiveQueueMessages(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details("Method not implemented!")
        raise NotImplementedError("Method not implemented!")

    def StreamQueueMessage(self, request_iterator, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details("Method not implemented!")
        raise NotImplementedError("Method not implemented!")

    def AckAllQueueMessages(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details("Method not implemented!")
        raise NotImplementedError("Method not implemented!")

    def Ping(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details("Method not implemented!")
        raise NotImplementedError("Method not implemented!")

    def QueuesDownstream(self, request_iterator, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details("Method not implemented!")
        raise NotImplementedError("Method not implemented!")

    def QueuesUpstream(self, request_iterator, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details("Method not implemented!")
        raise NotImplementedError("Method not implemented!")

    def QueuesInfo(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details("Method not implemented!")
        raise NotImplementedError("Method not implemented!")


def add_kubemqServicer_to_server(servicer, server):
    rpc_method_handlers = {
        "SendEvent": grpc.unary_unary_rpc_method_handler(
            servicer.SendEvent,
            request_deserializer=kubemq_dot_grpc_dot_kubemq__pb2.Event.FromString,
            response_serializer=kubemq_dot_grpc_dot_kubemq__pb2.Result.SerializeToString,
        ),
        "SendEventsStream": grpc.stream_stream_rpc_method_handler(
            servicer.SendEventsStream,
            request_deserializer=kubemq_dot_grpc_dot_kubemq__pb2.Event.FromString,
            response_serializer=kubemq_dot_grpc_dot_kubemq__pb2.Result.SerializeToString,
        ),
        "SubscribeToEvents": grpc.unary_stream_rpc_method_handler(
            servicer.SubscribeToEvents,
            request_deserializer=kubemq_dot_grpc_dot_kubemq__pb2.Subscribe.FromString,
            response_serializer=kubemq_dot_grpc_dot_kubemq__pb2.EventReceive.SerializeToString,
        ),
        "SubscribeToRequests": grpc.unary_stream_rpc_method_handler(
            servicer.SubscribeToRequests,
            request_deserializer=kubemq_dot_grpc_dot_kubemq__pb2.Subscribe.FromString,
            response_serializer=kubemq_dot_grpc_dot_kubemq__pb2.Request.SerializeToString,
        ),
        "SendRequest": grpc.unary_unary_rpc_method_handler(
            servicer.SendRequest,
            request_deserializer=kubemq_dot_grpc_dot_kubemq__pb2.Request.FromString,
            response_serializer=kubemq_dot_grpc_dot_kubemq__pb2.Response.SerializeToString,
        ),
        "SendResponse": grpc.unary_unary_rpc_method_handler(
            servicer.SendResponse,
            request_deserializer=kubemq_dot_grpc_dot_kubemq__pb2.Response.FromString,
            response_serializer=kubemq_dot_grpc_dot_kubemq__pb2.Empty.SerializeToString,
        ),
        "SendQueueMessage": grpc.unary_unary_rpc_method_handler(
            servicer.SendQueueMessage,
            request_deserializer=kubemq_dot_grpc_dot_kubemq__pb2.QueueMessage.FromString,
            response_serializer=kubemq_dot_grpc_dot_kubemq__pb2.SendQueueMessageResult.SerializeToString,
        ),
        "SendQueueMessagesBatch": grpc.unary_unary_rpc_method_handler(
            servicer.SendQueueMessagesBatch,
            request_deserializer=kubemq_dot_grpc_dot_kubemq__pb2.QueueMessagesBatchRequest.FromString,
            response_serializer=kubemq_dot_grpc_dot_kubemq__pb2.QueueMessagesBatchResponse.SerializeToString,
        ),
        "ReceiveQueueMessages": grpc.unary_unary_rpc_method_handler(
            servicer.ReceiveQueueMessages,
            request_deserializer=kubemq_dot_grpc_dot_kubemq__pb2.ReceiveQueueMessagesRequest.FromString,
            response_serializer=kubemq_dot_grpc_dot_kubemq__pb2.ReceiveQueueMessagesResponse.SerializeToString,
        ),
        "StreamQueueMessage": grpc.stream_stream_rpc_method_handler(
            servicer.StreamQueueMessage,
            request_deserializer=kubemq_dot_grpc_dot_kubemq__pb2.StreamQueueMessagesRequest.FromString,
            response_serializer=kubemq_dot_grpc_dot_kubemq__pb2.StreamQueueMessagesResponse.SerializeToString,
        ),
        "AckAllQueueMessages": grpc.unary_unary_rpc_method_handler(
            servicer.AckAllQueueMessages,
            request_deserializer=kubemq_dot_grpc_dot_kubemq__pb2.AckAllQueueMessagesRequest.FromString,
            response_serializer=kubemq_dot_grpc_dot_kubemq__pb2.AckAllQueueMessagesResponse.SerializeToString,
        ),
        "Ping": grpc.unary_unary_rpc_method_handler(
            servicer.Ping,
            request_deserializer=kubemq_dot_grpc_dot_kubemq__pb2.Empty.FromString,
            response_serializer=kubemq_dot_grpc_dot_kubemq__pb2.PingResult.SerializeToString,
        ),
        "QueuesDownstream": grpc.stream_stream_rpc_method_handler(
            servicer.QueuesDownstream,
            request_deserializer=kubemq_dot_grpc_dot_kubemq__pb2.QueuesDownstreamRequest.FromString,
            response_serializer=kubemq_dot_grpc_dot_kubemq__pb2.QueuesDownstreamResponse.SerializeToString,
        ),
        "QueuesUpstream": grpc.stream_stream_rpc_method_handler(
            servicer.QueuesUpstream,
            request_deserializer=kubemq_dot_grpc_dot_kubemq__pb2.QueuesUpstreamRequest.FromString,
            response_serializer=kubemq_dot_grpc_dot_kubemq__pb2.QueuesUpstreamResponse.SerializeToString,
        ),
        "QueuesInfo": grpc.unary_unary_rpc_method_handler(
            servicer.QueuesInfo,
            request_deserializer=kubemq_dot_grpc_dot_kubemq__pb2.QueuesInfoRequest.FromString,
            response_serializer=kubemq_dot_grpc_dot_kubemq__pb2.QueuesInfoResponse.SerializeToString,
        ),
    }
    generic_handler = grpc.method_handlers_generic_handler(
        "kubemq.kubemq", rpc_method_handlers
    )
    server.add_generic_rpc_handlers((generic_handler,))
    server.add_registered_method_handlers("kubemq.kubemq", rpc_method_handlers)


# This class is part of an EXPERIMENTAL API.
class kubemq(object):
    """Missing associated documentation comment in .proto file."""

    @staticmethod
    def SendEvent(
        request,
        target,
        options=(),
        channel_credentials=None,
        call_credentials=None,
        insecure=False,
        compression=None,
        wait_for_ready=None,
        timeout=None,
        metadata=None,
    ):
        return grpc.experimental.unary_unary(
            request,
            target,
            "/kubemq.kubemq/SendEvent",
            kubemq_dot_grpc_dot_kubemq__pb2.Event.SerializeToString,
            kubemq_dot_grpc_dot_kubemq__pb2.Result.FromString,
            options,
            channel_credentials,
            insecure,
            call_credentials,
            compression,
            wait_for_ready,
            timeout,
            metadata,
            _registered_method=True,
        )

    @staticmethod
    def SendEventsStream(
        request_iterator,
        target,
        options=(),
        channel_credentials=None,
        call_credentials=None,
        insecure=False,
        compression=None,
        wait_for_ready=None,
        timeout=None,
        metadata=None,
    ):
        return grpc.experimental.stream_stream(
            request_iterator,
            target,
            "/kubemq.kubemq/SendEventsStream",
            kubemq_dot_grpc_dot_kubemq__pb2.Event.SerializeToString,
            kubemq_dot_grpc_dot_kubemq__pb2.Result.FromString,
            options,
            channel_credentials,
            insecure,
            call_credentials,
            compression,
            wait_for_ready,
            timeout,
            metadata,
            _registered_method=True,
        )

    @staticmethod
    def SubscribeToEvents(
        request,
        target,
        options=(),
        channel_credentials=None,
        call_credentials=None,
        insecure=False,
        compression=None,
        wait_for_ready=None,
        timeout=None,
        metadata=None,
    ):
        return grpc.experimental.unary_stream(
            request,
            target,
            "/kubemq.kubemq/SubscribeToEvents",
            kubemq_dot_grpc_dot_kubemq__pb2.Subscribe.SerializeToString,
            kubemq_dot_grpc_dot_kubemq__pb2.EventReceive.FromString,
            options,
            channel_credentials,
            insecure,
            call_credentials,
            compression,
            wait_for_ready,
            timeout,
            metadata,
            _registered_method=True,
        )

    @staticmethod
    def SubscribeToRequests(
        request,
        target,
        options=(),
        channel_credentials=None,
        call_credentials=None,
        insecure=False,
        compression=None,
        wait_for_ready=None,
        timeout=None,
        metadata=None,
    ):
        return grpc.experimental.unary_stream(
            request,
            target,
            "/kubemq.kubemq/SubscribeToRequests",
            kubemq_dot_grpc_dot_kubemq__pb2.Subscribe.SerializeToString,
            kubemq_dot_grpc_dot_kubemq__pb2.Request.FromString,
            options,
            channel_credentials,
            insecure,
            call_credentials,
            compression,
            wait_for_ready,
            timeout,
            metadata,
            _registered_method=True,
        )

    @staticmethod
    def SendRequest(
        request,
        target,
        options=(),
        channel_credentials=None,
        call_credentials=None,
        insecure=False,
        compression=None,
        wait_for_ready=None,
        timeout=None,
        metadata=None,
    ):
        return grpc.experimental.unary_unary(
            request,
            target,
            "/kubemq.kubemq/SendRequest",
            kubemq_dot_grpc_dot_kubemq__pb2.Request.SerializeToString,
            kubemq_dot_grpc_dot_kubemq__pb2.Response.FromString,
            options,
            channel_credentials,
            insecure,
            call_credentials,
            compression,
            wait_for_ready,
            timeout,
            metadata,
            _registered_method=True,
        )

    @staticmethod
    def SendResponse(
        request,
        target,
        options=(),
        channel_credentials=None,
        call_credentials=None,
        insecure=False,
        compression=None,
        wait_for_ready=None,
        timeout=None,
        metadata=None,
    ):
        return grpc.experimental.unary_unary(
            request,
            target,
            "/kubemq.kubemq/SendResponse",
            kubemq_dot_grpc_dot_kubemq__pb2.Response.SerializeToString,
            kubemq_dot_grpc_dot_kubemq__pb2.Empty.FromString,
            options,
            channel_credentials,
            insecure,
            call_credentials,
            compression,
            wait_for_ready,
            timeout,
            metadata,
            _registered_method=True,
        )

    @staticmethod
    def SendQueueMessage(
        request,
        target,
        options=(),
        channel_credentials=None,
        call_credentials=None,
        insecure=False,
        compression=None,
        wait_for_ready=None,
        timeout=None,
        metadata=None,
    ):
        return grpc.experimental.unary_unary(
            request,
            target,
            "/kubemq.kubemq/SendQueueMessage",
            kubemq_dot_grpc_dot_kubemq__pb2.QueueMessage.SerializeToString,
            kubemq_dot_grpc_dot_kubemq__pb2.SendQueueMessageResult.FromString,
            options,
            channel_credentials,
            insecure,
            call_credentials,
            compression,
            wait_for_ready,
            timeout,
            metadata,
            _registered_method=True,
        )

    @staticmethod
    def SendQueueMessagesBatch(
        request,
        target,
        options=(),
        channel_credentials=None,
        call_credentials=None,
        insecure=False,
        compression=None,
        wait_for_ready=None,
        timeout=None,
        metadata=None,
    ):
        return grpc.experimental.unary_unary(
            request,
            target,
            "/kubemq.kubemq/SendQueueMessagesBatch",
            kubemq_dot_grpc_dot_kubemq__pb2.QueueMessagesBatchRequest.SerializeToString,
            kubemq_dot_grpc_dot_kubemq__pb2.QueueMessagesBatchResponse.FromString,
            options,
            channel_credentials,
            insecure,
            call_credentials,
            compression,
            wait_for_ready,
            timeout,
            metadata,
            _registered_method=True,
        )

    @staticmethod
    def ReceiveQueueMessages(
        request,
        target,
        options=(),
        channel_credentials=None,
        call_credentials=None,
        insecure=False,
        compression=None,
        wait_for_ready=None,
        timeout=None,
        metadata=None,
    ):
        return grpc.experimental.unary_unary(
            request,
            target,
            "/kubemq.kubemq/ReceiveQueueMessages",
            kubemq_dot_grpc_dot_kubemq__pb2.ReceiveQueueMessagesRequest.SerializeToString,
            kubemq_dot_grpc_dot_kubemq__pb2.ReceiveQueueMessagesResponse.FromString,
            options,
            channel_credentials,
            insecure,
            call_credentials,
            compression,
            wait_for_ready,
            timeout,
            metadata,
            _registered_method=True,
        )

    @staticmethod
    def StreamQueueMessage(
        request_iterator,
        target,
        options=(),
        channel_credentials=None,
        call_credentials=None,
        insecure=False,
        compression=None,
        wait_for_ready=None,
        timeout=None,
        metadata=None,
    ):
        return grpc.experimental.stream_stream(
            request_iterator,
            target,
            "/kubemq.kubemq/StreamQueueMessage",
            kubemq_dot_grpc_dot_kubemq__pb2.StreamQueueMessagesRequest.SerializeToString,
            kubemq_dot_grpc_dot_kubemq__pb2.StreamQueueMessagesResponse.FromString,
            options,
            channel_credentials,
            insecure,
            call_credentials,
            compression,
            wait_for_ready,
            timeout,
            metadata,
            _registered_method=True,
        )

    @staticmethod
    def AckAllQueueMessages(
        request,
        target,
        options=(),
        channel_credentials=None,
        call_credentials=None,
        insecure=False,
        compression=None,
        wait_for_ready=None,
        timeout=None,
        metadata=None,
    ):
        return grpc.experimental.unary_unary(
            request,
            target,
            "/kubemq.kubemq/AckAllQueueMessages",
            kubemq_dot_grpc_dot_kubemq__pb2.AckAllQueueMessagesRequest.SerializeToString,
            kubemq_dot_grpc_dot_kubemq__pb2.AckAllQueueMessagesResponse.FromString,
            options,
            channel_credentials,
            insecure,
            call_credentials,
            compression,
            wait_for_ready,
            timeout,
            metadata,
            _registered_method=True,
        )

    @staticmethod
    def Ping(
        request,
        target,
        options=(),
        channel_credentials=None,
        call_credentials=None,
        insecure=False,
        compression=None,
        wait_for_ready=None,
        timeout=None,
        metadata=None,
    ):
        return grpc.experimental.unary_unary(
            request,
            target,
            "/kubemq.kubemq/Ping",
            kubemq_dot_grpc_dot_kubemq__pb2.Empty.SerializeToString,
            kubemq_dot_grpc_dot_kubemq__pb2.PingResult.FromString,
            options,
            channel_credentials,
            insecure,
            call_credentials,
            compression,
            wait_for_ready,
            timeout,
            metadata,
            _registered_method=True,
        )

    @staticmethod
    def QueuesDownstream(
        request_iterator,
        target,
        options=(),
        channel_credentials=None,
        call_credentials=None,
        insecure=False,
        compression=None,
        wait_for_ready=None,
        timeout=None,
        metadata=None,
    ):
        return grpc.experimental.stream_stream(
            request_iterator,
            target,
            "/kubemq.kubemq/QueuesDownstream",
            kubemq_dot_grpc_dot_kubemq__pb2.QueuesDownstreamRequest.SerializeToString,
            kubemq_dot_grpc_dot_kubemq__pb2.QueuesDownstreamResponse.FromString,
            options,
            channel_credentials,
            insecure,
            call_credentials,
            compression,
            wait_for_ready,
            timeout,
            metadata,
            _registered_method=True,
        )

    @staticmethod
    def QueuesUpstream(
        request_iterator,
        target,
        options=(),
        channel_credentials=None,
        call_credentials=None,
        insecure=False,
        compression=None,
        wait_for_ready=None,
        timeout=None,
        metadata=None,
    ):
        return grpc.experimental.stream_stream(
            request_iterator,
            target,
            "/kubemq.kubemq/QueuesUpstream",
            kubemq_dot_grpc_dot_kubemq__pb2.QueuesUpstreamRequest.SerializeToString,
            kubemq_dot_grpc_dot_kubemq__pb2.QueuesUpstreamResponse.FromString,
            options,
            channel_credentials,
            insecure,
            call_credentials,
            compression,
            wait_for_ready,
            timeout,
            metadata,
            _registered_method=True,
        )

    @staticmethod
    def QueuesInfo(
        request,
        target,
        options=(),
        channel_credentials=None,
        call_credentials=None,
        insecure=False,
        compression=None,
        wait_for_ready=None,
        timeout=None,
        metadata=None,
    ):
        return grpc.experimental.unary_unary(
            request,
            target,
            "/kubemq.kubemq/QueuesInfo",
            kubemq_dot_grpc_dot_kubemq__pb2.QueuesInfoRequest.SerializeToString,
            kubemq_dot_grpc_dot_kubemq__pb2.QueuesInfoResponse.FromString,
            options,
            channel_credentials,
            insecure,
            call_credentials,
            compression,
            wait_for_ready,
            timeout,
            metadata,
            _registered_method=True,
        )



import time
import threading
from random import random

from kubemq.queues import *


def worker(message: QueueMessageReceived):
    print(f"Worker received message: {message.body.decode('utf-8')}\n")
    time.sleep(random())  # Simulate processing time
    message.re_queue("q2")


def main():
    with Client(address="localhost:50000") as client:
        # Send 10 messages
        for i in range(10):
            send_result = client.send_queues_message(
                QueueMessage(
                    channel="q1",
                    body=f"Message {i + 1}".encode("utf-8"),
                    metadata="some-metadata",
                )
            )
            print(f"Queue Message Sent: {send_result}")

        # Wait for 1 second
        time.sleep(1)

        # Receive 10 messages
        result = client.receive_queues_messages(
            channel="q1",
            max_messages=10,
            wait_timeout_in_seconds=10,
            auto_ack=False,
        )
        if result.is_error:
            print(f"{result.error}")
            return

        # Create 10 threads, each processing one message
        threads = []
        for message in result.messages:
            thread = threading.Thread(target=worker, args=(message,))
            threads.append(thread)

        # Start all threads simultaneously
        for thread in threads:
            thread.start()

        # Wait for all threads to complete
        for thread in threads:
            thread.join()
        time.sleep(1)


if __name__ == "__main__":
    main()



from pydantic import BaseModel, field_validator
from typing import Callable, Optional
from kubemq.grpc import Subscribe
from kubemq.common.subscribe_type import SubscribeType
from kubemq.cq.query_message_received import QueryMessageReceived


class QueriesSubscription(BaseModel):
    """
    QueriesSubscription class represents a subscription to receive query messages from a channel.

    Attributes:
        channel (str): The name of the channel to subscribe to.
        group (Optional[str]): The optional name of the group to subscribe to.
        on_receive_query_callback (Callable[[QueryMessageReceived], None]): The callback function to be called when a query message is received.
        on_error_callback (Optional[Callable[[str], None]]): The callback function to be called when an error occurs.
    """

    channel: str
    group: Optional[str] = None
    on_receive_query_callback: Callable[[QueryMessageReceived], None]
    on_error_callback: Optional[Callable[[str], None]] = None

    model_config = {"arbitrary_types_allowed": True}

    @field_validator("channel")
    def channel_must_exist(cls, v: str) -> str:
        if not v:
            raise ValueError("query subscription must have a channel.")
        return v

    @field_validator("on_receive_query_callback")
    def callback_must_exist(cls, v: Callable) -> Callable:
        if not callable(v):
            raise ValueError(
                "query subscription must have a on_receive_query_callback function."
            )
        return v

    def raise_on_receive_message(self, received_query: QueryMessageReceived) -> None:
        """Raises the on_receive_query_callback with the received query message."""
        self.on_receive_query_callback(received_query)

    def raise_on_error(self, msg: str) -> None:
        """Raises the on_error_callback with the specified error message."""
        if self.on_error_callback:
            self.on_error_callback(msg)

    def encode(self, client_id: str = "") -> Subscribe:
        """Encodes the query subscription into a Subscribe message."""
        request = Subscribe()
        request.Channel = self.channel
        request.Group = self.group or ""
        request.ClientID = client_id
        request.SubscribeTypeData = SubscribeType.Queries.value
        return request

    def __repr__(self) -> str:
        """Returns a string representation of the QueriesSubscription object."""
        return f"QueriesSubscription: channel={self.channel}, group={self.group}"

    @classmethod
    def create(
        cls,
        channel: str,
        group: Optional[str] = None,
        on_receive_query_callback: Callable[[QueryMessageReceived], None] = None,
        on_error_callback: Optional[Callable[[str], None]] = None,
    ) -> "QueriesSubscription":
        """
        Creates a new QueriesSubscription instance.

        This method provides backwards compatibility with the original constructor.
        """
        return cls(
            channel=channel,
            group=group,
            on_receive_query_callback=on_receive_query_callback,
            on_error_callback=on_error_callback,
        )



from kubemq.queues import *


def delete_queues_channel(channelName):
    try:
        with Client(address="localhost:50000", client_id="delete_example") as client:
            client.delete_queues_channel(channelName)
            print(f"Queues channel: {channelName} deleted successfully.")
    except Exception as e:
        print(f"Error deleting Queues Channel: {channelName},  {e}")


if __name__ == "__main__":
    delete_queues_channel("new_queues_channel")



from .queues_poll_request import *
from .queues_message import *
from .queues_message_received import *
from .queues_poll_request import *
from .queues_poll_response import *
from .downstream_receiver import *
from .queues_send_result import *
from .upstream_sender import *
from .queues_messages_waiting_pulled import *
from .client import *



import threading
import logging
import time
import grpc
from kubemq.transport.connection import Connection
from kubemq.transport.interceptors import AuthInterceptors
from kubemq.transport.tls_config import TlsConfig
import kubemq.grpc.kubemq_pb2_grpc as kubemq_pb2_grpc
from kubemq.grpc import Empty


def _get_ssl_credentials(tls_config: TlsConfig):
    from kubemq.transport.transport import _get_ssl_credentials as get_creds

    return get_creds(tls_config)


def _get_call_options(connection: Connection):
    from kubemq.transport.transport import _get_call_options as get_opts

    return get_opts(connection)


class ConnectionState:
    """
    Class for managing the shared connection state across components.
    """

    def __init__(self):
        self.lock = threading.Lock()
        self.is_connected = True

    def set_connected(self, value: bool):
        with self.lock:
            old_state = self.is_connected
            self.is_connected = value
            return old_state != value  # Return True if state changed

    def is_accepting_requests(self) -> bool:
        with self.lock:
            return self.is_connected


class ChannelManager:
    """
    Centralized manager for gRPC channel creation and reconnection.

    This class coordinates channel access and recreation across multiple components.
    """

    def __init__(self, connection: Connection, logger: logging.Logger):
        self._opts = connection.complete()
        self._connection = connection
        self._channel = None
        self._client = None
        self._channel_lock = threading.Lock()
        self.connection_state = ConnectionState()
        self.logger = logger
        self._registered_clients = []
        self._initialize_channel()

    def _initialize_channel(self):
        """Initialize the gRPC channel and client stub"""
        with self._channel_lock:
            try:
                auth_interceptor = AuthInterceptors(self._opts.auth_token)
                interceptors = [auth_interceptor]
                credentials = (
                    _get_ssl_credentials(self._opts.tls)
                    if self._opts.tls.enabled
                    else None
                )
                self._channel = (
                    grpc.secure_channel(
                        self._opts.address,
                        credentials,
                        options=_get_call_options(self._opts),
                    )
                    if credentials
                    else grpc.insecure_channel(
                        self._opts.address, options=_get_call_options(self._opts)
                    )
                )
                self._channel = grpc.intercept_channel(self._channel, *interceptors)
                self._client = kubemq_pb2_grpc.kubemqStub(self._channel)
                self._test_connection()
                self.connection_state.set_connected(True)
            except Exception as ex:
                self.logger.error(f"Failed to initialize channel: {str(ex)}")
                self.connection_state.set_connected(False)
                raise ex

    def _test_connection(self):
        """Test the connection to the server"""
        try:
            self._client.Ping(Empty())
            return True
        except Exception as e:
            self.logger.error(f"Connection test failed: {str(e)}")
            return False

    def register_client(self, client_ref):
        """Register a client to be notified of channel updates"""
        self._registered_clients.append(client_ref)

    def get_client(self) -> kubemq_pb2_grpc.kubemqStub:
        """Get the current gRPC client stub"""
        with self._channel_lock:
            return self._client

    def is_channel_healthy(self) -> bool:
        """Check if the channel is in a healthy state"""
        try:
            return self._test_connection()
        except Exception:
            return False

    def recreate_channel(self) -> kubemq_pb2_grpc.kubemqStub:
        """
        Recreate the gRPC channel and client after a connection failure

        Returns:
            kubemq_pb2_grpc.kubemqStub: New client instance
        """
        with self._channel_lock:
            self.logger.info("Starting channel recreation process")
            self.connection_state.set_connected(False)

            # Check if auto-reconnect is disabled
            if self._connection.disable_auto_reconnect:
                self.logger.warning(
                    "Auto-reconnect is disabled, not attempting to recreate channel"
                )
                raise ConnectionError("Auto-reconnect is disabled by configuration")

            # Close existing channel if exists
            if self._channel is not None:
                try:
                    self._channel.close()
                except Exception as e:
                    self.logger.warning(f"Error closing existing channel: {str(e)}")
                self._channel = None
                self._client = None

            # Use fixed reconnection interval from client configuration
            reconnect_seconds = self._connection.reconnect_interval_seconds
            if reconnect_seconds <= 0:
                reconnect_seconds = 1  # Default to 1 second if not specified

            self.logger.info(f"Waiting {reconnect_seconds} seconds before reconnection")
            time.sleep(reconnect_seconds)

            # Recreate channel with existing credentials and options
            try:
                auth_interceptor = AuthInterceptors(self._opts.auth_token)
                interceptors = [auth_interceptor]
                credentials = (
                    _get_ssl_credentials(self._opts.tls)
                    if self._opts.tls.enabled
                    else None
                )

                self._channel = (
                    grpc.secure_channel(
                        self._opts.address,
                        credentials,
                        options=_get_call_options(self._opts),
                    )
                    if credentials
                    else grpc.insecure_channel(
                        self._opts.address, options=_get_call_options(self._opts)
                    )
                )
                self._channel = grpc.intercept_channel(self._channel, *interceptors)
                self._client = kubemq_pb2_grpc.kubemqStub(self._channel)

                # Test the connection
                if self._test_connection():
                    self.logger.info(
                        "Successfully recreated gRPC channel and verified connection"
                    )
                    self.connection_state.set_connected(True)
                else:
                    self.logger.warning("Channel recreated but connection test failed")
                    # We'll keep the connected state as False
            except Exception as e:
                self.logger.error(f"Failed to recreate channel: {str(e)}")
                raise e

            return self._client

    def close(self):
        """Close the channel and clean up resources"""
        with self._channel_lock:
            if self._channel is not None:
                try:
                    self._channel.close()
                except Exception as e:
                    self.logger.warning(f"Error while closing channel: {str(e)}")
                self._channel = None
                self._client = None
            self.connection_state.set_connected(False)



from enum import Enum


class SubscribeType(Enum):
    """Type of subscription operation pattern"""

    SubscribeTypeUndefined = 0
    """Default"""

    Events = 1
    """PubSub event"""

    EventsStore = 2
    """PubSub event with persistence"""

    Commands = 3
    """ReqRep perform action"""

    Queries = 4
    """ReqRep return data"""



import uuid
import grpc
from kubemq.common.exceptions import *
from kubemq.common.helpers import decode_grpc_error
from kubemq.grpc import Request
from kubemq.common.channel_stats import *

requests_channel = "kubemq.cluster.internal.requests"


def create_channel_request(
    transport, client_id, channel_name, channel_type
) -> [bool, None]:
    """

    This method creates a request to create a channel in the Kubemq server.

    Parameters:
    - transport: The transport object that handles the communication with the Kubemq server.
    - client_id: The ID of the client.
    - channel_name: The name of the channel to create.
    - channel_type: The type of the channel.

    Returns:
    - bool: True if the channel creation was successful.
    - None: If an error occurred during the channel creation.

    Raises:
    - CreateChannelError: If the channel creation request was executed but with an error response from the server.
    - GRPCError: If there was an error with the gRPC communication with the Kubemq server.

    """
    try:
        request = Request(
            RequestID=str(uuid.uuid4()),
            RequestTypeData=2,
            Metadata="create-channel",
            Channel=requests_channel,
            ClientID=client_id,
            Tags={
                "channel_type": channel_type,
                "channel": channel_name,
                "client_id": client_id,
            },
            Timeout=10 * 1000,
        )
        response = transport.kubemq_client().SendRequest(request)
        if response:
            if response.Executed:
                return True
            else:
                raise CreateChannelError(response.Error)
    except grpc.RpcError as e:
        raise GRPCError(decode_grpc_error(e))


def delete_channel_request(
    transport, client_id, channel_name, channel_type
) -> [bool, None]:
    """

    This method is used to send a delete channel request to the Kubemq server. It deletes a channel with the specified name and type.

    Parameters:
    - transport: The transport object used for communication with the Kubemq server.
    - client_id: The client ID associated with the request.
    - channel_name: The name of the channel to be deleted.
    - channel_type: The type of the channel to be deleted.

    Returns:
    - If the delete channel request is executed successfully, it returns True.
    - If there is an error during the execution of the delete channel request, it raises a DeleteChannelError with the corresponding error message.

    Raises:
    - GRPCError: If there is a GRPC error during the process.

    """
    try:
        request = Request(
            RequestID=str(uuid.uuid4()),
            RequestTypeData=2,
            Metadata="delete-channel",
            Channel=requests_channel,
            ClientID=client_id,
            Tags={
                "channel_type": channel_type,
                "channel": channel_name,
                "client_id": client_id,
            },
            Timeout=10 * 1000,
        )
        response = transport.kubemq_client().SendRequest(request)
        if response:
            if response.Executed:
                return True
            else:
                raise DeleteChannelError(response.Error)
    except grpc.RpcError as e:
        raise GRPCError(decode_grpc_error(e))


def list_queues_channels(transport, client_id, channel_search) -> List[QueuesChannel]:
    """

    List Queues Channels

    This method is used to list the queues channels from the Kubemq server. It takes the following parameters:

    Parameters:
    - `transport` : The transport object used to communicate with the Kubemq server.
    - `client_id` : The client ID of the client making the request.
    - `channel_search` : The search query string to filter the channels. (optional)

    Returns:
    - `List[QueuesChannel]` : A list of QueuesChannel objects representing the queues channels.

    Raises:
    - `ListChannelsError` : If the client fails to list channels, this error will be raised.
    - `GRPCError` : If there is an error while communicating with the Kubemq server, this error will be raised.

    Example Usage:

    ```python
    transport = Transport()
    client_id = "my_client_id"

    # List all queues channels
    queues_channels = list_queues_channels(transport, client_id)

    # List queues channels with a specific search query
    search_query = "search query"
    queues_channels = list_queues_channels(transport, client_id, search_query)
    ```
    """
    try:
        request = Request(
            RequestID=str(uuid.uuid4()),
            RequestTypeData=2,
            Metadata="list-channels",
            Channel=requests_channel,
            ClientID=client_id,
            Tags={"channel_type": "queues", "channel_search": channel_search},
            Timeout=10 * 1000,
        )
        response = transport.kubemq_client().SendRequest(request)
        if response:
            if response.Executed:
                return decode_queues_channel_list(response.Body)
            else:
                self.logger.error(
                    f"Client failed to list {channel_type} channels, error: {response.Error}"
                )
                raise ListChannelsError(response.Error)
    except grpc.RpcError as e:
        raise GRPCError(decode_grpc_error(e))


def list_pubsub_channels(
    transport, client_id, channel_type: str, channel_search
) -> List[PubSubChannel]:
    """

    This method is used to retrieve a list of PubSub channels based on the specified parameters.

    Parameters:
    - transport: The transport object used for communication.
    - client_id: The ID of the client making the request.
    - channel_type (str): The type of PubSub channel to filter the list by.
    - channel_search: A search parameter to further filter the list of channels.

    Returns:
    - List[PubSubChannel]: A list of PubSubChannel objects representing the channels matching the specified parameters.

    Raises:
    - ListChannelsError: If the request fails or is not executed successfully.
    - GRPCError: If a gRPC error occurs during the execution of the request.

    """
    try:
        request = Request(
            RequestID=str(uuid.uuid4()),
            RequestTypeData=2,
            Metadata="list-channels",
            Channel=requests_channel,
            ClientID=client_id,
            Tags={"channel_type": channel_type, "channel_search": channel_search},
            Timeout=10 * 1000,
        )
        response = transport.kubemq_client().SendRequest(request)
        if response:
            if response.Executed:
                return decode_pub_sub_channel_list(response.Body)
            else:
                self.logger.error(
                    f"Client failed to list {channel_type} channels, error: {response.Error}"
                )
                raise ListChannelsError(response.Error)
    except grpc.RpcError as e:
        raise GRPCError(decode_grpc_error(e))


def list_cq_channels(
    transport, client_id, channel_type: str, channel_search
) -> List[CQChannel]:
    """

    Method: list_cq_channels

    Parameters:
    - transport: The transport object used for communication with the Kubemq server.
    - client_id: The ID of the client making the request.
    - channel_type (str): The type of channel to list.
    - channel_search: The search keyword for filtering the channels.

    Returns:
    - List[CQChannel]: A list of CQChannel objects representing the channels that match the search criteria.

    Raises:
    - ListChannelsError: If the client fails to list the channels.
    - GRPCError: If a gRPC error occurs during the request.

    """
    try:
        request = Request(
            RequestID=str(uuid.uuid4()),
            RequestTypeData=2,
            Metadata="list-channels",
            Channel=requests_channel,
            ClientID=client_id,
            Tags={"channel_type": channel_type, "channel_search": channel_search},
            Timeout=10 * 1000,
        )
        response = transport.kubemq_client().SendRequest(request)
        if response:
            if response.Executed:
                return decode_cq_channel_list(response.Body)
            else:
                self.logger.error(
                    f"Client failed to list {channel_type} channels, error: {response.Error}"
                )
                raise ListChannelsError(response.Error)
    except grpc.RpcError as e:
        raise GRPCError(decode_grpc_error(e))



from kubemq.pubsub import Client


def main():
    try:
        client = Client(address="localhost:50000")
        server_info = client.ping()
        print(server_info)
    except Exception as e:
        print(e)
        return


if __name__ == "__main__":
    main()



from pydantic import BaseModel, Field, field_validator, model_validator, ConfigDict
from typing import Optional, ClassVar
from kubemq.transport.keep_alive import KeepAliveConfig
from kubemq.transport.tls_config import TlsConfig


class Connection(BaseModel):
    DEFAULT_MAX_SEND_SIZE: ClassVar[int] = 1024 * 1024 * 100  # 100MB
    DEFAULT_MAX_RCV_SIZE: ClassVar[int] = 1024 * 1024 * 100  # 100MB
    DEFAULT_RECONNECT_INTERVAL_SECONDS: ClassVar[int] = 1

    address: str = Field(default="")
    client_id: str = Field(default="")
    auth_token: str = Field(default="")
    max_send_size: int = Field(default=DEFAULT_MAX_SEND_SIZE)
    max_receive_size: int = Field(default=DEFAULT_MAX_RCV_SIZE)
    disable_auto_reconnect: bool = Field(default=False)
    reconnect_interval_seconds: int = Field(default=DEFAULT_RECONNECT_INTERVAL_SECONDS)
    tls: TlsConfig = Field(default_factory=TlsConfig)
    keep_alive: KeepAliveConfig = Field(default_factory=KeepAliveConfig)
    log_level: Optional[int] = Field(default=None)

    model_config = ConfigDict(validate_assignment=True, arbitrary_types_allowed=True)

    @field_validator("address")
    def validate_address(cls, v: str) -> str:
        if not v:
            raise ValueError("Connection must have an address")
        return v

    @field_validator("client_id")
    def validate_client_id(cls, v: str) -> str:
        if not v:
            raise ValueError("Connection must have a client_id")
        return v

    @field_validator("max_send_size", "max_receive_size", "reconnect_interval_seconds")
    def validate_positive_values(cls, v: int, info) -> int:
        if v < 0:
            raise ValueError(f"{info.field_name} must be greater than or equal to 0")
        return v

    @model_validator(mode="after")
    def validate_tls_and_keep_alive(self) -> "Connection":
        # TLS and KeepAlive configs are automatically validated by Pydantic
        return self

    def get_reconnect_delay(self) -> int:
        return self.reconnect_interval_seconds

    def set_log_level(self, value: int) -> "Connection":
        self.log_level = value
        return self

    def complete(self) -> "Connection":
        if self.max_send_size == 0:
            self.max_send_size = self.DEFAULT_MAX_SEND_SIZE
        if self.max_receive_size == 0:
            self.max_receive_size = self.DEFAULT_MAX_RCV_SIZE
        if self.reconnect_interval_seconds == 0:
            self.reconnect_interval_seconds = self.DEFAULT_RECONNECT_INTERVAL_SECONDS
        return self



import time
from kubemq.cq import *


def main():
    try:
        client = Client(address="localhost:50000")

        def on_receive_command(request: CommandMessageReceived):
            try:
                print(f"Id:{request.id}, Body:{request.body.decode('utf-8')}")
                response = CommandResponseMessage(
                    command_received=request,
                    is_executed=True,
                )
                client.send_response_message(response)
            except Exception as e:
                print(e)

        def on_error_handler(err: str):
            print(f"{err}")

        client.subscribe_to_commands(
            subscription=CommandsSubscription(
                channel="c1",
                group="",
                on_receive_command_callback=on_receive_command,
                on_error_callback=on_error_handler,
            ),
            cancel=CancellationToken(),
        )
        time.sleep(1)
        response = client.send_command_request(
            CommandMessage(
                channel="c1",
                body=b"hello kubemq, please reply to me!",
                timeout_in_seconds=10,
            )
        )
        print(
            f"Request Execution: {response.is_executed}, Executed at: {response.timestamp}, Error: {response.error}"
        )
    except Exception as e:
        print(e)
        return


if __name__ == "__main__":
    main()
